@@ -93,8 +93,7 @@ static QCString convertIndexWordToAnchor(const QCString &word)
           (c >= 'A' && c <= 'Z') || // ALPHA
           (c >= '0' && c <= '9') || // DIGIT
           c == '-' ||
-          c == '.' ||
-          c == '_'
+          c == '.'
          )
       {
         result += c;
@@ -102,7 +101,7 @@ static QCString convertIndexWordToAnchor(const QCString &word)
       else
       {
         char enc[4];
-        enc[0] = ';';
+        enc[0] = '_';
         enc[1] = hex[(c & 0xf0) >> 4];
         enc[2] = hex[c & 0xf];
         enc[3] = 0;
@@ -130,6 +129,7 @@ static bool mustBeOutsideParagraph(const DocNodeVariant &n)
                                  /* \dotfile */     DocDotFile,
                                  /* \mscfile */     DocMscFile,
                                  /* \diafile */     DocDiaFile,
+                                 /* \plantumlfile */ DocPlantUmlFile,
                                  /* <details> */    DocHtmlDetails,
                                  /* <summary> */    DocHtmlSummary,
                                                     DocIncOperator >(n))
@@ -226,64 +226,6 @@ static bool isInvisibleNode(const DocNodeVariant &node)
   return false;
 }
 
-static void mergeHtmlAttributes(const HtmlAttribList &attribs, HtmlAttribList &mergeInto)
-{
-  for (const auto &att : attribs)
-  {
-    auto it = std::find_if(std::begin(mergeInto),std::end(mergeInto),
-                           [&att](const auto &opt) { return opt.name==att.name; });
-    if (it!=std::end(mergeInto)) // attribute name already in mergeInto
-    {
-       it->value = it->value + " " + att.value;
-    }
-    else // attribute name not yet in mergeInto
-    {
-      mergeInto.push_back(att);
-    }
-  }
-}
-
-static QCString htmlAttribsToString(const HtmlAttribList &attribs, QCString *pAltValue = nullptr)
-{
-  QCString result;
-  for (const auto &att : attribs)
-  {
-    if (!att.value.isEmpty())  // ignore attribute without values as they
-                                // are not XHTML compliant, with the exception
-				// of the alt attribute with the img tag
-    {
-      if (att.name=="alt" && pAltValue) // optionally return the value of alt separately
-                                         // need to convert <img> to <object> for SVG images,
-                                         // which do not support the alt attribute
-      {
-        *pAltValue = att.value;
-      }
-      else
-      {
-        result+=" ";
-        result+=att.name;
-        result+="=\""+convertToXML(att.value)+"\"";
-      }
-    }
-    else if (att.name=="open")
-    {
-        // The open attribute is a boolean attribute.
-        // Specifies that the details should be visible (open) to the user
-        // As it is a boolean attribute the initialisation value is of no interest
-        result+=" ";
-        result+=att.name;
-        result+="=\"true\"";
-    }
-    else if (att.name=="nowrap") // In XHTML, attribute minimization is forbidden, and the nowrap attribute must be defined as <td nowrap="nowrap">.
-    {
-        result+=" ";
-        result+=att.name;
-        result+="=\"nowrap\"";
-    }
-  }
-  return result;
-}
-
 static QCString makeShortName(const QCString &name)
 {
   QCString shortName = name;
@@ -468,14 +410,14 @@ void HtmlDocVisitor::operator()(const DocURL &u)
 void HtmlDocVisitor::operator()(const DocLineBreak &br)
 {
   if (m_hide) return;
-  m_t << "<br "<< htmlAttribsToString(br.attribs()) << " />\n";
+  m_t << "<br "<< br.attribs().toString() << " />\n";
 }
 
 void HtmlDocVisitor::operator()(const DocHorRuler &hr)
 {
   if (m_hide) return;
   forceEndParagraph(hr);
-  m_t << "<hr "<< htmlAttribsToString(hr.attribs()) << " />\n";
+  m_t << "<hr "<< hr.attribs().toString() << " />\n";
   forceStartParagraph(hr);
 }
 
@@ -485,40 +427,67 @@ void HtmlDocVisitor::operator()(const DocStyleChange &s)
   switch (s.style())
   {
     case DocStyleChange::Bold:
-      if (s.enable()) m_t << "<b" << htmlAttribsToString(s.attribs()) << ">";      else m_t << "</b>";
+      if (s.enable()) m_t << "<b" << s.attribs().toString() << ">";      else m_t << "</b>";
       break;
     case DocStyleChange::S:
-      if (s.enable()) m_t << "<s" << htmlAttribsToString(s.attribs()) << ">";      else m_t << "</s>";
+      if (s.enable()) m_t << "<s" << s.attribs().toString() << ">";      else m_t << "</s>";
       break;
     case DocStyleChange::Strike:
-      if (s.enable()) m_t << "<strike" << htmlAttribsToString(s.attribs()) << ">";      else m_t << "</strike>";
+      if (s.enable()) m_t << "<strike" << s.attribs().toString() << ">"; else m_t << "</strike>";
       break;
     case DocStyleChange::Del:
-      if (s.enable()) m_t << "<del" << htmlAttribsToString(s.attribs()) << ">";      else m_t << "</del>";
+      if (s.enable()) m_t << "<del" << s.attribs().toString() << ">";    else m_t << "</del>";
       break;
     case DocStyleChange::Underline:
-      if (s.enable()) m_t << "<u" << htmlAttribsToString(s.attribs()) << ">";      else m_t << "</u>";
+      if (s.enable()) m_t << "<u" << s.attribs().toString() << ">";      else m_t << "</u>";
       break;
     case DocStyleChange::Ins:
-      if (s.enable()) m_t << "<ins" << htmlAttribsToString(s.attribs()) << ">";      else m_t << "</ins>";
+      if (s.enable()) m_t << "<ins" << s.attribs().toString() << ">";    else m_t << "</ins>";
       break;
     case DocStyleChange::Italic:
-      if (s.enable()) m_t << "<em" << htmlAttribsToString(s.attribs()) << ">";     else m_t << "</em>";
+      if (s.enable())
+      {
+        auto attribs = s.attribs();
+        if (s.tagName()=="a")
+        {
+          attribs.mergeAttribute("class","arg");
+        }
+        m_t << "<em" << attribs.toString() << ">";
+      }
+      else
+      {
+        m_t << "</em>";
+      }
+      break;
+    case DocStyleChange::Kbd:
+      if (s.enable()) m_t << "<kbd" << s.attribs().toString() << ">";    else m_t << "</kbd>";
       break;
     case DocStyleChange::Code:
-      if (s.enable()) m_t << "<code" << htmlAttribsToString(s.attribs()) << ">";   else m_t << "</code>";
+      if (s.enable())
+      {
+        auto attribs = s.attribs();
+        if (s.tagName()=="p")
+        {
+          attribs.mergeAttribute("class","param");
+        }
+        m_t << "<code" << attribs.toString() << ">";
+      }
+      else
+      {
+        m_t << "</code>";
+      }
       break;
     case DocStyleChange::Subscript:
-      if (s.enable()) m_t << "<sub" << htmlAttribsToString(s.attribs()) << ">";    else m_t << "</sub>";
+      if (s.enable()) m_t << "<sub" << s.attribs().toString() << ">"; else m_t << "</sub>";
       break;
     case DocStyleChange::Superscript:
-      if (s.enable()) m_t << "<sup" << htmlAttribsToString(s.attribs()) << ">";    else m_t << "</sup>";
+      if (s.enable()) m_t << "<sup" << s.attribs().toString() << ">"; else m_t << "</sup>";
       break;
     case DocStyleChange::Center:
       if (s.enable())
       {
         forceEndParagraph(s);
-        m_t << "<center" << htmlAttribsToString(s.attribs()) << ">";
+        m_t << "<center" << s.attribs().toString() << ">";
       }
       else
       {
@@ -527,16 +496,16 @@ void HtmlDocVisitor::operator()(const DocStyleChange &s)
       }
       break;
     case DocStyleChange::Small:
-      if (s.enable()) m_t << "<small" << htmlAttribsToString(s.attribs()) << ">";  else m_t << "</small>";
+      if (s.enable()) m_t << "<small" << s.attribs().toString() << ">";  else m_t << "</small>";
       break;
     case DocStyleChange::Cite:
-      if (s.enable()) m_t << "<cite" << htmlAttribsToString(s.attribs()) << ">";  else m_t << "</cite>";
+      if (s.enable()) m_t << "<cite" << s.attribs().toString() << ">";  else m_t << "</cite>";
       break;
     case DocStyleChange::Preformatted:
       if (s.enable())
       {
         forceEndParagraph(s);
-        m_t << "<pre" << htmlAttribsToString(s.attribs()) << ">";
+        m_t << "<pre" << s.attribs().toString() << ">";
         m_insidePre=TRUE;
       }
       else
@@ -550,7 +519,7 @@ void HtmlDocVisitor::operator()(const DocStyleChange &s)
       if (s.enable())
       {
         forceEndParagraph(s);
-        m_t << "<div" << htmlAttribsToString(s.attribs()) << ">";
+        m_t << "<div" << s.attribs().toString() << ">";
       }
       else
       {
@@ -559,7 +528,7 @@ void HtmlDocVisitor::operator()(const DocStyleChange &s)
       }
       break;
     case DocStyleChange::Span:
-      if (s.enable()) m_t << "<span" << htmlAttribsToString(s.attribs()) << ">";  else m_t << "</span>";
+      if (s.enable()) m_t << "<span" << s.attribs().toString() << ">";  else m_t << "</span>";
       break;
   }
 }
@@ -584,15 +553,16 @@ void HtmlDocVisitor::operator()(const DocVerbatim &s)
                                         s.context(),
                                         s.text(),
                                         langExt,
+                                        Config_getBool(STRIP_CODE_COMMENTS),
                                         s.isExample(),
                                         s.exampleFile(),
-                                        nullptr,     // fileDef
-                                        -1,    // startLine
-                                        -1,    // endLine
-                                        FALSE, // inlineFragment
-                                        nullptr,     // memberDef
-                                        TRUE,  // show line numbers
-                                        m_ctx  // search context
+                                        nullptr, // fileDef
+                                        -1,      // startLine
+                                        -1,      // endLine
+                                        true,    // inlineFragment
+                                        nullptr, // memberDef
+                                        true,    // show line numbers
+                                        m_ctx    // search context
                                        );
       m_ci.endCodeFragment("DoxyCode");
       forceStartParagraph(s);
@@ -706,7 +676,7 @@ void HtmlDocVisitor::operator()(const DocVerbatim &s)
         }
         QCString baseName = PlantumlManager::instance().writePlantUMLSource(
                                     htmlOutput,s.exampleFile(),
-                                    s.text(),format,s.engine(),s.srcFile(),s.srcLine());
+                                    s.text(),format,s.engine(),s.srcFile(),s.srcLine(),true);
         m_t << "<div class=\"plantumlgraph\">\n";
         writePlantUMLFile(baseName,s.relPath(),s.context(),s.srcFile(),s.srcLine());
         visitCaption(m_t, s);
@@ -720,7 +690,7 @@ void HtmlDocVisitor::operator()(const DocVerbatim &s)
 void HtmlDocVisitor::operator()(const DocAnchor &anc)
 {
   if (m_hide) return;
-  m_t << "<a class=\"anchor\" id=\"" << anc.anchor() << "\"" << htmlAttribsToString(anc.attribs()) << "></a>";
+  m_t << "<a class=\"anchor\" id=\"" << anc.anchor() << "\"" << anc.attribs().toString() << "></a>";
 }
 
 void HtmlDocVisitor::operator()(const DocInclude &inc)
@@ -736,6 +706,7 @@ void HtmlDocVisitor::operator()(const DocInclude &inc)
                                         inc.context(),
                                         inc.text(),
                                         langExt,
+                                        inc.stripCodeComments(),
                                         inc.isExample(),
                                         inc.exampleFile(),
                                         nullptr,     // fileDef
@@ -759,15 +730,16 @@ void HtmlDocVisitor::operator()(const DocInclude &inc)
                                            inc.context(),
                                            inc.text(),
                                            langExt,
+                                           inc.stripCodeComments(),
                                            inc.isExample(),
                                            inc.exampleFile(),
                                            fd.get(),   // fileDef,
-                                           -1,    // start line
-                                           -1,    // end line
-                                           FALSE, // inline fragment
-                                           nullptr,     // memberDef
-                                           TRUE,  // show line numbers
-                                           m_ctx  // search context
+                                           -1,         // start line
+                                           -1,         // end line
+                                           true,       // inline fragment
+                                           nullptr,    // memberDef
+                                           true,       // show line numbers
+                                           m_ctx       // search context
                                            );
          m_ci.endCodeFragment("DoxyCode");
          forceStartParagraph(inc);
@@ -796,7 +768,6 @@ void HtmlDocVisitor::operator()(const DocInclude &inc)
       forceStartParagraph(inc);
       break;
     case DocInclude::Snippet:
-    case DocInclude::SnippetTrimLeft:
     case DocInclude::SnippetWithLines:
       forceEndParagraph(inc);
       m_ci.startCodeFragment("DoxyCode");
@@ -805,7 +776,8 @@ void HtmlDocVisitor::operator()(const DocInclude &inc)
                                        inc.blockId(),
                                        inc.context(),
                                        inc.type()==DocInclude::SnippetWithLines,
-                                       inc.type()==DocInclude::SnippetTrimLeft
+                                       inc.trimLeft(),
+                                       inc.stripCodeComments()
                                       );
       m_ci.endCodeFragment("DoxyCode");
       forceStartParagraph(inc);
@@ -843,15 +815,16 @@ void HtmlDocVisitor::operator()(const DocIncOperator &op)
                                 op.context(),
                                 op.text(),
                                 langExt,
+                                op.stripCodeComments(),
                                 op.isExample(),
                                 op.exampleFile(),
                                 fd.get(),     // fileDef
                                 op.line(),    // startLine
-                                -1,    // endLine
-                                FALSE, // inline fragment
-                                nullptr,     // memberDef
+                                -1,           // endLine
+                                true,         // inline fragment
+                                nullptr,      // memberDef
                                 op.showLineNo(),  // show line numbers
-                                m_ctx  // search context
+                                m_ctx         // search context
                                );
     }
     pushHidden(m_hide);
@@ -1345,9 +1318,9 @@ void HtmlDocVisitor::operator()(const DocPara &p)
   if (needsTagBefore)
   {
     if (contexts(t))
-      m_t << "<p class=\"" << contexts(t) << "\"" << htmlAttribsToString(p.attribs()) << ">";
+      m_t << "<p class=\"" << contexts(t) << "\"" << p.attribs().toString() << ">";
     else
-      m_t << "<p" << htmlAttribsToString(p.attribs()) << ">";
+      m_t << "<p" << p.attribs().toString() << ">";
   }
 
   visitChildren(p);
@@ -1504,11 +1477,11 @@ void HtmlDocVisitor::operator()(const DocHtmlList &s)
   forceEndParagraph(s);
   if (s.type()==DocHtmlList::Ordered)
   {
-    m_t << "<ol" << htmlAttribsToString(s.attribs());
+    m_t << "<ol" << s.attribs().toString();
   }
   else
   {
-    m_t << "<ul" << htmlAttribsToString(s.attribs());
+    m_t << "<ul" << s.attribs().toString();
   }
   m_t << ">\n";
   visitChildren(s);
@@ -1527,7 +1500,7 @@ void HtmlDocVisitor::operator()(const DocHtmlList &s)
 void HtmlDocVisitor::operator()(const DocHtmlListItem &i)
 {
   if (m_hide) return;
-  m_t << "<li" << htmlAttribsToString(i.attribs()) << ">";
+  m_t << "<li" << i.attribs().toString() << ">";
   if (!i.isPreformatted()) m_t << "\n";
   visitChildren(i);
   m_t << "</li>\n";
@@ -1537,7 +1510,7 @@ void HtmlDocVisitor::operator()(const DocHtmlDescList &dl)
 {
   if (m_hide) return;
   forceEndParagraph(dl);
-  m_t << "<dl" << htmlAttribsToString(dl.attribs()) << ">\n";
+  m_t << "<dl" << dl.attribs().toString() << ">\n";
   visitChildren(dl);
   m_t << "</dl>\n";
   forceStartParagraph(dl);
@@ -1546,7 +1519,7 @@ void HtmlDocVisitor::operator()(const DocHtmlDescList &dl)
 void HtmlDocVisitor::operator()(const DocHtmlDescTitle &dt)
 {
   if (m_hide) return;
-  m_t << "<dt" << htmlAttribsToString(dt.attribs()) << ">";
+  m_t << "<dt" << dt.attribs().toString() << ">";
   visitChildren(dt);
   m_t << "</dt>\n";
 }
@@ -1554,7 +1527,7 @@ void HtmlDocVisitor::operator()(const DocHtmlDescTitle &dt)
 void HtmlDocVisitor::operator()(const DocHtmlDescData &dd)
 {
   if (m_hide) return;
-  m_t << "<dd" << htmlAttribsToString(dd.attribs()) << ">";
+  m_t << "<dd" << dd.attribs().toString() << ">";
   visitChildren(dd);
   m_t << "</dd>\n";
 }
@@ -1574,14 +1547,14 @@ void HtmlDocVisitor::operator()(const DocHtmlTable &t)
     }
   }
 
-  QCString attrs = htmlAttribsToString(t.attribs());
+  QCString attrs = t.attribs().toString();
   if (attrs.isEmpty())
   {
     m_t << "<table class=\"doxtable\">\n";
   }
   else
   {
-    m_t << "<table" << htmlAttribsToString(t.attribs()) << ">\n";
+    m_t << "<table" << t.attribs().toString() << ">\n";
   }
   if (t.caption())
   {
@@ -1595,7 +1568,7 @@ void HtmlDocVisitor::operator()(const DocHtmlTable &t)
 void HtmlDocVisitor::operator()(const DocHtmlRow &tr)
 {
   if (m_hide) return;
-  m_t << "<tr" << htmlAttribsToString(tr.attribs()) << ">\n";
+  m_t << "<tr" << tr.attribs().toString() << ">\n";
   visitChildren(tr);
   m_t << "</tr>\n";
 }
@@ -1605,11 +1578,11 @@ void HtmlDocVisitor::operator()(const DocHtmlCell &c)
   if (m_hide) return;
   if (c.isHeading())
   {
-    m_t << "<th" << htmlAttribsToString(c.attribs()) << ">";
+    m_t << "<th" << c.attribs().toString() << ">";
   }
   else
   {
-    m_t << "<td" << htmlAttribsToString(c.attribs()) << ">";
+    m_t << "<td" << c.attribs().toString() << ">";
   }
   visitChildren(c);
   if (c.isHeading()) m_t << "</th>"; else m_t << "</td>";
@@ -1618,7 +1591,7 @@ void HtmlDocVisitor::operator()(const DocHtmlCell &c)
 void HtmlDocVisitor::operator()(const DocHtmlCaption &c)
 {
   if (m_hide) return;
-  m_t << "<caption" << htmlAttribsToString(c.attribs()) << ">";
+  m_t << "<caption" << c.attribs().toString() << ">";
   visitChildren(c);
   m_t << "</caption>\n";
 }
@@ -1640,7 +1613,7 @@ void HtmlDocVisitor::operator()(const DocHRef &href)
   {
     QCString url = correctURL(href.url(),href.relPath());
     m_t << "<a href=\"" << convertToHtml(url)  << "\""
-        << htmlAttribsToString(href.attribs()) << ">";
+        << href.attribs().toString() << ">";
   }
   visitChildren(href);
   m_t << "</a>";
@@ -1649,7 +1622,7 @@ void HtmlDocVisitor::operator()(const DocHRef &href)
 void HtmlDocVisitor::operator()(const DocHtmlSummary &s)
 {
   if (m_hide) return;
-  m_t << "<summary " << htmlAttribsToString(s.attribs()) << ">\n";
+  m_t << "<summary " << s.attribs().toString() << ">\n";
   visitChildren(s);
   m_t << "</summary>\n";
 }
@@ -1658,7 +1631,7 @@ void HtmlDocVisitor::operator()(const DocHtmlDetails &d)
 {
   if (m_hide) return;
   forceEndParagraph(d);
-  m_t << "<details " << htmlAttribsToString(d.attribs()) << ">\n";
+  m_t << "<details " << d.attribs().toString() << ">\n";
   auto summary = d.summary();
   if (summary)
   {
@@ -1673,7 +1646,7 @@ void HtmlDocVisitor::operator()(const DocHtmlHeader &header)
 {
   if (m_hide) return;
   forceEndParagraph(header);
-  m_t << "<h" << header.level() << htmlAttribsToString(header.attribs()) << ">";
+  m_t << "<h" << header.level() << header.attribs().toString() << ">";
   visitChildren(header);
   m_t << "</h" << header.level() << ">\n";
   forceStartParagraph(header);
@@ -1705,17 +1678,13 @@ void HtmlDocVisitor::operator()(const DocImage &img)
     }
     // 16 cases: url.isEmpty() | typeSVG | inlineImage | img.hasCaption()
 
-    HtmlAttribList extraAttribs;
+    HtmlAttribList attribs = img.attribs();
     if (typeSVG)
     {
-      HtmlAttrib opt;
-      opt.name  = "style";
-      opt.value = "pointer-events: none;";
-      extraAttribs.push_back(opt);
+      attribs.mergeAttribute("style","pointer-events: none;");
     }
     QCString alt;
-    mergeHtmlAttributes(img.attribs(),extraAttribs);
-    QCString attrs = htmlAttribsToString(extraAttribs,&alt);
+    QCString attrs = attribs.toString(&alt);
     QCString src;
     if (url.isEmpty())
     {
@@ -1852,6 +1821,39 @@ void HtmlDocVisitor::operator()(const DocDiaFile &df)
   forceStartParagraph(df);
 }
 
+void HtmlDocVisitor::operator()(const DocPlantUmlFile &df)
+{
+  if (m_hide) return;
+  if (!Config_getBool(DOT_CLEANUP)) copyFile(df.file(),Config_getString(HTML_OUTPUT)+"/"+stripPath(df.file()));
+  forceEndParagraph(df);
+  QCString htmlOutput = Config_getString(HTML_OUTPUT);
+  QCString imgExt = getDotImageExtension();
+  PlantumlManager::OutputFormat format = PlantumlManager::PUML_BITMAP;	// default : PUML_BITMAP
+  if (imgExt=="svg")
+  {
+    format = PlantumlManager::PUML_SVG;
+  }
+  std::string inBuf;
+  readInputFile(df.file(),inBuf);
+  QCString baseName = PlantumlManager::instance().writePlantUMLSource(
+                                    htmlOutput,QCString(),
+                                    inBuf.c_str(),format,QCString(),df.srcFile(),df.srcLine(),false);
+  baseName=makeBaseName(baseName);
+  m_t << "<div class=\"plantumlgraph\">\n";
+  writePlantUMLFile(baseName,df.relPath(),QCString(),df.srcFile(),df.srcLine());
+  if (df.hasCaption())
+  {
+    m_t << "<div class=\"caption\">\n";
+  }
+  visitChildren(df);
+  if (df.hasCaption())
+  {
+    m_t << "</div>\n";
+  }
+  m_t << "</div>\n";
+  forceStartParagraph(df);
+}
+
 void HtmlDocVisitor::operator()(const DocLink &lnk)
 {
   if (m_hide) return;
@@ -2045,7 +2047,7 @@ void HtmlDocVisitor::operator()(const DocHtmlBlockQuote &b)
 {
   if (m_hide) return;
   forceEndParagraph(b);
-  m_t << "<blockquote class=\"doxtable\"" << htmlAttribsToString(b.attribs()) << ">\n";
+  m_t << "<blockquote class=\"doxtable\"" << b.attribs().toString() << ">\n";
   visitChildren(b);
   m_t << "</blockquote>\n";
   forceStartParagraph(b);
@@ -2206,7 +2208,7 @@ void HtmlDocVisitor::writeDotFile(const QCString &fn,const QCString &relPath,
   QCString baseName=makeBaseName(fn);
   baseName.prepend("dot_");
   QCString outDir = Config_getString(HTML_OUTPUT);
-  writeDotGraphFromFile(fn,outDir,baseName,GOF_BITMAP,srcFile,srcLine);
+  writeDotGraphFromFile(fn,outDir,baseName,GraphOutputFormat::BITMAP,srcFile,srcLine);
   writeDotImageMapFromFile(m_t,fn,outDir,relPath,baseName,context,-1,srcFile,srcLine);
 }
 
@@ -2217,9 +2219,9 @@ void HtmlDocVisitor::writeMscFile(const QCString &fileName,const QCString &relPa
   baseName.prepend("msc_");
   QCString outDir = Config_getString(HTML_OUTPUT);
   QCString imgExt = getDotImageExtension();
-  MscOutputFormat mscFormat = MSC_BITMAP;
+  MscOutputFormat mscFormat = MscOutputFormat::BITMAP;
   if ("svg" == imgExt)
-    mscFormat = MSC_SVG;
+    mscFormat = MscOutputFormat::SVG;
   writeMscGraphFromFile(fileName,outDir,baseName,mscFormat,srcFile,srcLine);
   writeMscImageMapFromFile(m_t,fileName,outDir,relPath,baseName,context,mscFormat,srcFile,srcLine);
 }
@@ -2230,7 +2232,7 @@ void HtmlDocVisitor::writeDiaFile(const QCString &fileName, const QCString &relP
   QCString baseName=makeBaseName(fileName);
   baseName.prepend("dia_");
   QCString outDir = Config_getString(HTML_OUTPUT);
-  writeDiaGraphFromFile(fileName,outDir,baseName,DIA_BITMAP,srcFile,srcLine);
+  writeDiaGraphFromFile(fileName,outDir,baseName,DiaOutputFormat::BITMAP,srcFile,srcLine);
 
   m_t << "<img src=\"" << relPath << baseName << ".png" << "\" />\n";
 }
