@@ -20,6 +20,7 @@
 #include <vector>
 #include <map>
 #include "qcstring.h"
+#include "construct.h"
 
 class Definition;
 class OutputList;
@@ -33,6 +34,7 @@ enum class IndexSection
   isTitlePageAuthor,
   isMainPage,
   isModuleIndex,
+  isTopicIndex,
   isDirIndex,
   isNamespaceIndex,
   isConceptIndex,
@@ -41,6 +43,7 @@ enum class IndexSection
   isFileIndex,
   isPageIndex,
   isModuleDocumentation,
+  isTopicDocumentation,
   isDirDocumentation,
   isNamespaceDocumentation,
   isClassDocumentation,
@@ -58,6 +61,7 @@ enum class HighlightedItem
   Main,
   Modules,
   Namespaces,
+  Topics,
   ClassHierarchy,
   InterfaceHierarchy,
   ExceptionHierarchy,
@@ -72,6 +76,7 @@ enum class HighlightedItem
   AnnotatedExceptions,
   Files,
   NamespaceMembers,
+  ModuleMembers,
   Functions,
   Globals,
   Pages,
@@ -85,7 +90,8 @@ enum class HighlightedItem
   StructVisible,
   ExceptionVisible,
   NamespaceVisible,
-  FileVisible
+  FileVisible,
+  ModuleVisible
 };
 
 // Note: we can't use enum class for the enums below as they are also used as an array index,
@@ -141,6 +147,20 @@ namespace NamespaceMemberHighlight
   };
 } // namespace NamespaceMemberHighlight
 
+namespace ModuleMemberHighlight
+{
+  enum Enum : int
+  {
+    All = 0,
+    Functions,
+    Variables,
+    Typedefs,
+    Enums,
+    EnumValues,
+    Total
+  };
+} // namespace ModuleMemberHighlight
+
 class Index
 {
   public:
@@ -153,6 +173,7 @@ class Index
     void addClassMemberNameToIndex(const MemberDef *md);
     void addFileMemberNameToIndex(const MemberDef *md);
     void addNamespaceMemberNameToIndex(const MemberDef *md);
+    void addModuleMemberNameToIndex(const MemberDef *md);
     void sortMemberIndexLists();
 
     // ---- getters
@@ -170,6 +191,7 @@ class Index
     int numDocumentedGroups() const;
     int numDocumentedNamespaces() const;
     int numDocumentedConcepts() const;
+    int numDocumentedModules() const;
     int numIndexedPages() const;
     int numDocumentedFiles() const;
     int numDocumentedPages() const;
@@ -177,26 +199,31 @@ class Index
     int numDocumentedClassMembers(ClassMemberHighlight::Enum e) const;
     int numDocumentedFileMembers(FileMemberHighlight::Enum e) const;
     int numDocumentedNamespaceMembers(NamespaceMemberHighlight::Enum e) const;
+    int numDocumentedModuleMembers(ModuleMemberHighlight::Enum e) const;
     MemberIndexMap isClassIndexLetterUsed(ClassMemberHighlight::Enum e) const;
     MemberIndexMap isFileIndexLetterUsed(FileMemberHighlight::Enum e) const;
     MemberIndexMap isNamespaceIndexLetterUsed(NamespaceMemberHighlight::Enum e) const;
+    MemberIndexMap isModuleIndexLetterUsed(ModuleMemberHighlight::Enum e) const;
 
   private:
     void resetDocumentedClassMembers(int i);
     void resetDocumentedFileMembers(int i);
     void resetDocumentedNamespaceMembers(int i);
+    void resetDocumentedModuleMembers(int i);
     void incrementDocumentedClassMembers(int i,const std::string &letter,const MemberDef *md);
     void incrementDocumentedFileMembers(int i,const std::string &letter,const MemberDef *md);
     void incrementDocumentedNamespaceMembers(int i,const std::string &letter,const MemberDef *md);
+    void incrementDocumentedModuleMembers(int i,const std::string &letter,const MemberDef *md);
     Index();
     ~Index();
+    NON_COPYABLE(Index)
     struct Private;
     std::unique_ptr<Private> p;
 };
 
 void writeGraphInfo(OutputList &ol);
 void writeIndexHierarchy(OutputList &ol);
-void startTitle(OutputList &ol,const QCString &fileName,const DefinitionMutable *def=0);
+void startTitle(OutputList &ol,const QCString &fileName,const DefinitionMutable *def=nullptr);
 void endTitle(OutputList &ol,const QCString &fileName,const QCString &name);
 void startFile(OutputList &ol,const QCString &name,const QCString &manName,
                const QCString &title,HighlightedItem hli=HighlightedItem::None,
