@@ -20,130 +20,112 @@
 
 #include "qcstring.h"
 #include "docvisitor.h"
+#include "docnode.h"
 
+class OutputCodeList;
 class LatexCodeGenerator;
 class TextStream;
 
+enum class TexOrPdf
+{
+   NO,  //!< not called through texorpdf
+   TEX, //!< called through texorpdf as TeX (first) part
+   PDF, //!< called through texorpdf as PDF (second) part
+};
+
 /*! @brief Concrete visitor implementation for LaTeX output. */
 class LatexDocVisitor : public DocVisitor
 {
   public:
-    LatexDocVisitor(TextStream &t,LatexCodeGenerator &ci,
-                    const QCString &langExt,bool insideTabbing);
+    LatexDocVisitor(TextStream &t,OutputCodeList &ci,LatexCodeGenerator &lcg,
+                    const QCString &langExt, int hierarchyLevel = 0);
 
     //--------------------------------------
     // visitor functions for leaf nodes
     //--------------------------------------
 
-    void visit(DocWord *);
-    void visit(DocLinkedWord *);
-    void visit(DocWhiteSpace *);
-    void visit(DocSymbol *);
-    void visit(DocEmoji *);
-    void visit(DocURL *);
-    void visit(DocLineBreak *);
-    void visit(DocHorRuler *);
-    void visit(DocStyleChange *);
-    void visit(DocVerbatim *);
-    void visit(DocAnchor *);
-    void visit(DocInclude *);
-    void visit(DocIncOperator *);
-    void visit(DocFormula *);
-    void visit(DocIndexEntry *);
-    void visit(DocSimpleSectSep *);
-    void visit(DocCite *);
+    void operator()(const DocWord &);
+    void operator()(const DocLinkedWord &);
+    void operator()(const DocWhiteSpace &);
+    void operator()(const DocSymbol &);
+    void operator()(const DocEmoji &);
+    void operator()(const DocURL &);
+    void operator()(const DocLineBreak &);
+    void operator()(const DocHorRuler &);
+    void operator()(const DocStyleChange &);
+    void operator()(const DocVerbatim &);
+    void operator()(const DocAnchor &);
+    void operator()(const DocInclude &);
+    void operator()(const DocIncOperator &);
+    void operator()(const DocFormula &);
+    void operator()(const DocIndexEntry &);
+    void operator()(const DocSimpleSectSep &);
+    void operator()(const DocCite &);
+    void operator()(const DocSeparator &);
 
     //--------------------------------------
     // visitor functions for compound nodes
     //--------------------------------------
 
-    void visitPre(DocAutoList *);
-    void visitPost(DocAutoList *);
-    void visitPre(DocAutoListItem *);
-    void visitPost(DocAutoListItem *);
-    void visitPre(DocPara *);
-    void visitPost(DocPara *);
-    void visitPre(DocRoot *);
-    void visitPost(DocRoot *);
-    void visitPre(DocSimpleSect *);
-    void visitPost(DocSimpleSect *);
-    void visitPre(DocTitle *);
-    void visitPost(DocTitle *);
-    void visitPre(DocSimpleList *);
-    void visitPost(DocSimpleList *);
-    void visitPre(DocSimpleListItem *);
-    void visitPost(DocSimpleListItem *);
-    void visitPre(DocSection *s);
-    void visitPost(DocSection *);
-    void visitPre(DocHtmlList *s);
-    void visitPost(DocHtmlList *s);
-    void visitPre(DocHtmlListItem *);
-    void visitPost(DocHtmlListItem *);
-    //void visitPre(DocHtmlPre *);
-    //void visitPost(DocHtmlPre *);
-    void visitPre(DocHtmlDescList *);
-    void visitPost(DocHtmlDescList *);
-    void visitPre(DocHtmlDescTitle *);
-    void visitPost(DocHtmlDescTitle *);
-    void visitPre(DocHtmlDescData *);
-    void visitPost(DocHtmlDescData *);
-    void visitPre(DocHtmlTable *t);
-    void visitPost(DocHtmlTable *t);
-    void visitPre(DocHtmlCaption *);
-    void visitPost(DocHtmlCaption *);
-    void visitPre(DocHtmlRow *);
-    void visitPost(DocHtmlRow *) ;
-    void visitPre(DocHtmlCell *);
-    void visitPost(DocHtmlCell *);
-    void visitPre(DocInternal *);
-    void visitPost(DocInternal *);
-    void visitPre(DocHRef *);
-    void visitPost(DocHRef *);
-    void visitPre(DocHtmlHeader *);
-    void visitPost(DocHtmlHeader *) ;
-    void visitPre(DocImage *);
-    void visitPost(DocImage *);
-    void visitPre(DocDotFile *);
-    void visitPost(DocDotFile *);
-    void visitPre(DocMscFile *);
-    void visitPost(DocMscFile *);
-    void visitPre(DocDiaFile *);
-    void visitPost(DocDiaFile *);
-    void visitPre(DocLink *lnk);
-    void visitPost(DocLink *);
-    void visitPre(DocRef *ref);
-    void visitPost(DocRef *);
-    void visitPre(DocSecRefItem *);
-    void visitPost(DocSecRefItem *);
-    void visitPre(DocSecRefList *);
-    void visitPost(DocSecRefList *);
-    void visitPre(DocParamSect *);
-    void visitPost(DocParamSect *);
-    void visitPre(DocParamList *);
-    void visitPost(DocParamList *);
-    void visitPre(DocXRefItem *);
-    void visitPost(DocXRefItem *);
-    void visitPre(DocInternalRef *);
-    void visitPost(DocInternalRef *);
-    void visitPre(DocText *);
-    void visitPost(DocText *);
-    void visitPre(DocHtmlBlockQuote *);
-    void visitPost(DocHtmlBlockQuote *);
-    void visitPre(DocVhdlFlow *);
-    void visitPost(DocVhdlFlow *);
-    void visitPre(DocParBlock *);
-    void visitPost(DocParBlock *);
+    void operator()(const DocAutoList &);
+    void operator()(const DocAutoListItem &);
+    void operator()(const DocPara &);
+    void operator()(const DocRoot &);
+    void operator()(const DocSimpleSect &);
+    void operator()(const DocTitle &);
+    void operator()(const DocSimpleList &);
+    void operator()(const DocSimpleListItem &);
+    void operator()(const DocSection &s);
+    void operator()(const DocHtmlList &s);
+    void operator()(const DocHtmlListItem &);
+    void operator()(const DocHtmlDescList &);
+    void operator()(const DocHtmlDescTitle &);
+    void operator()(const DocHtmlDescData &);
+    void operator()(const DocHtmlTable &t);
+    void operator()(const DocHtmlCaption &);
+    void operator()(const DocHtmlRow &);
+    void operator()(const DocHtmlCell &);
+    void operator()(const DocInternal &);
+    void operator()(const DocHRef &);
+    void operator()(const DocHtmlSummary &);
+    void operator()(const DocHtmlDetails &);
+    void operator()(const DocHtmlHeader &);
+    void operator()(const DocImage &);
+    void operator()(const DocDotFile &);
+    void operator()(const DocMscFile &);
+    void operator()(const DocDiaFile &);
+    void operator()(const DocPlantUmlFile &);
+    void operator()(const DocLink &lnk);
+    void operator()(const DocRef &ref);
+    void operator()(const DocSecRefItem &);
+    void operator()(const DocSecRefList &);
+    void operator()(const DocParamSect &);
+    void operator()(const DocParamList &);
+    void operator()(const DocXRefItem &);
+    void operator()(const DocInternalRef &);
+    void operator()(const DocText &);
+    void operator()(const DocHtmlBlockQuote &);
+    void operator()(const DocVhdlFlow &);
+    void operator()(const DocParBlock &);
 
   private:
+    template<class T>
+    void visitChildren(const T &t)
+    {
+      for (const auto &child : t.children())
+      {
+        std::visit(*this, child);
+      }
+    }
 
     struct ActiveRowSpan
     {
-      ActiveRowSpan(DocHtmlCell *c,int rs,int cs,int col)
+      ActiveRowSpan(const DocHtmlCell &c,size_t rs,size_t cs,size_t col)
         : cell(c), rowSpan(rs), colSpan(cs), column(col) {}
-      DocHtmlCell *cell;
-      int rowSpan;
-      int colSpan;
-      int column;
+      const DocHtmlCell &cell;
+      size_t rowSpan;
+      size_t colSpan;
+      size_t column;
     };
 
     typedef std::vector<ActiveRowSpan> RowSpanList;
@@ -152,11 +134,11 @@ class LatexDocVisitor : public DocVisitor
     // helper functions
     //--------------------------------------
 
-    void filter(const QCString &str);
+    void filter(const QCString &str, const bool retainNewLine = false);
     void startLink(const QCString &ref,const QCString &file,
-                   const QCString &anchor,bool refToTable=FALSE);
+                   const QCString &anchor,bool refToTable=false,bool refToSection=false);
     void endLink(const QCString &ref,const QCString &file,
-                 const QCString &anchor,bool refToTable=FALSE);
+                 const QCString &anchor,bool refToTable=false,bool refToSection=false, SectionType sectionType = SectionType::Anchor);
     QCString escapeMakeIndexChars(const char *s);
     void startDotFile(const QCString &fileName,const QCString &width,
                       const QCString &height, bool hasCaption,
@@ -167,33 +149,45 @@ class LatexDocVisitor : public DocVisitor
                       const QCString &height, bool hasCaption,
                       const QCString &srcFile,int srcLine);
     void endMscFile(bool hasCaption);
-    void writeMscFile(const QCString &fileName, DocVerbatim *s);
+    void writeMscFile(const QCString &fileName, const DocVerbatim &s);
 
     void startDiaFile(const QCString &fileName,const QCString &width,
                       const QCString &height, bool hasCaption,
                       const QCString &srcFile,int srcLine);
     void endDiaFile(bool hasCaption);
-    void writeDiaFile(const QCString &fileName, DocVerbatim *s);
-    void writePlantUMLFile(const QCString &fileName, DocVerbatim *s);
+    void writeDiaFile(const QCString &fileName, const DocVerbatim &s);
+    void writePlantUMLFile(const QCString &fileName, const DocVerbatim &s);
+    void startPlantUmlFile(const QCString &fileName,const QCString &width,
+                      const QCString &height, bool hasCaption,
+                      const QCString &srcFile,int srcLine);
+    void endPlantUmlFile(bool hasCaption);
+
+    void visitCaption(const DocNodeList &children);
+
+    void incIndentLevel();
+    void decIndentLevel();
+    int indentLevel() const;
+    const char *getSectionName(int level) const;
 
     //--------------------------------------
     // state variables
     //--------------------------------------
 
     TextStream &m_t;
-    LatexCodeGenerator &m_ci;
+    OutputCodeList &m_ci;
+    LatexCodeGenerator &m_lcg;
     bool m_insidePre;
     bool m_insideItem;
     bool m_hide;
-    bool m_hideCaption;
-    bool m_insideTabbing;
     QCString m_langExt;
+    int m_hierarchyLevel;
+    TexOrPdf m_texOrPdf = TexOrPdf::NO;
 
     struct TableState
     {
       RowSpanList rowSpans;
-      int  numCols = 0;
-      int currentColumn = 0;
+      size_t numCols = 0;
+      size_t currentColumn = 0;
       bool inRowSpan = false;
       bool inColSpan = false;
       bool firstRow = false;
@@ -201,27 +195,37 @@ class LatexDocVisitor : public DocVisitor
     std::stack<TableState> m_tableStateStack; // needed for nested tables
     RowSpanList m_emptyRowSpanList;
 
+    static const int maxIndentLevels = 13;
+    int m_indentLevel = 0;
+
+    struct LatexListItemInfo
+    {
+      bool isEnum = false;
+    };
+
+    LatexListItemInfo m_listItemInfo[maxIndentLevels];
+
     void pushTableState()
     {
-      m_tableStateStack.push(TableState());
+      m_tableStateStack.emplace();
     }
     void popTableState()
     {
       m_tableStateStack.pop();
     }
-    int currentColumn() const
+    size_t currentColumn() const
     {
       return !m_tableStateStack.empty() ? m_tableStateStack.top().currentColumn : 0;
     }
-    void setCurrentColumn(int col)
+    void setCurrentColumn(size_t col)
     {
       if (!m_tableStateStack.empty()) m_tableStateStack.top().currentColumn = col;
     }
-    int numCols() const
+    size_t numCols() const
     {
       return !m_tableStateStack.empty() ? m_tableStateStack.top().numCols : 0;
     }
-    void setNumCols(int num)
+    void setNumCols(size_t num)
     {
       if (!m_tableStateStack.empty()) m_tableStateStack.top().numCols = num;
     }
@@ -263,5 +267,4 @@ class LatexDocVisitor : public DocVisitor
     }
 
 };
-
 #endif
