@@ -27,10 +27,9 @@
 #include "htmlentity.h"
 #include "emoji.h"
 #include "fileinfo.h"
+#include "codefragment.h"
 
-ManListItemInfo man_listItemInfo[man_maxIndentLevels];
-
-ManDocVisitor::ManDocVisitor(TextStream &t,CodeOutputInterface &ci,
+ManDocVisitor::ManDocVisitor(TextStream &t,OutputCodeList &ci,
                              const QCString &langExt)
   : m_t(t), m_ci(ci), m_insidePre(FALSE), m_hide(FALSE), m_firstCol(FALSE),
     m_indent(0), m_langExt(langExt)
@@ -148,8 +147,9 @@ void ManDocVisitor::operator()(const DocStyleChange &s)
       if (s.enable()) m_t << "\\fI";     else m_t << "\\fP";
       m_firstCol=FALSE;
       break;
+    case DocStyleChange::Kbd:
     case DocStyleChange::Code:
-      if (s.enable()) m_t << "\\fC";   else m_t << "\\fP";
+      if (s.enable()) m_t << "\\fR";   else m_t << "\\fP";
       m_firstCol=FALSE;
       break;
     case DocStyleChange::Subscript:
@@ -208,6 +208,7 @@ void ManDocVisitor::operator()(const DocVerbatim &s)
       m_t << ".nf\n";
       getCodeParser(lang).parseCode(m_ci,s.context(),s.text(),
                                         langExt,
+                                        Config_getBool(STRIP_CODE_COMMENTS),
                                         s.isExample(),s.exampleFile());
       if (!m_firstCol) m_t << "\n";
       m_t << ".fi\n";
@@ -218,7 +219,7 @@ void ManDocVisitor::operator()(const DocVerbatim &s)
       filter(s.text());
       break;
     case DocVerbatim::JavaDocCode:
-      m_t << "\\fC\n";
+      m_t << "\\fR\n";
       filter(s.text());
       m_t << "\\fP\n";
       break;
@@ -265,21 +266,20 @@ void ManDocVisitor::operator()(const DocInclude &inc)
          m_t << ".PP\n";
          m_t << ".nf\n";
          FileInfo cfi( inc.file().str() );
-         FileDef *fd = createFileDef( cfi.dirPath(), cfi.fileName() );
+         auto fd = createFileDef( cfi.dirPath(), cfi.fileName() );
          getCodeParser(inc.extension()).parseCode(m_ci,inc.context(),
                                            inc.text(),
                                            langExt,
+                                           inc.stripCodeComments(),
                                            inc.isExample(),
                                            inc.exampleFile(),
-                                           fd,   // fileDef,
+                                           fd.get(), // fileDef,
                                            -1,    // start line
                                            -1,    // end line
                                            FALSE, // inline fragment
-                                           0,     // memberDef
+                                           nullptr,     // memberDef
                                            TRUE
 					   );
-         delete fd;
-         if (!m_firstCol) m_t << "\n";
          m_t << ".fi\n";
          m_t << ".PP\n";
          m_firstCol=TRUE;
@@ -292,16 +292,16 @@ void ManDocVisitor::operator()(const DocInclude &inc)
       getCodeParser(inc.extension()).parseCode(m_ci,inc.context(),
                                         inc.text(),
                                         langExt,
+                                        inc.stripCodeComments(),
                                         inc.isExample(),
                                         inc.exampleFile(),
-                                        0,     // fileDef
+                                        nullptr,     // fileDef
                                         -1,    // startLine
                                         -1,    // endLine
                                         TRUE,  // inlineFragment
-                                        0,     // memberDef
+                                        nullptr,     // memberDef
                                         FALSE
 				       );
-      if (!m_firstCol) m_t << "\n";
       m_t << ".fi\n";
       m_t << ".PP\n";
       m_firstCol=TRUE;
@@ -328,53 +328,22 @@ void ManDocVisitor::operator()(const DocInclude &inc)
       m_firstCol=TRUE;
       break;
     case DocInclude::Snippet:
+    case DocInclude::SnippetWithLines:
       if (!m_firstCol) m_t << "\n";
       m_t << ".PP\n";
       m_t << ".nf\n";
-      getCodeParser(inc.extension()).parseCode(m_ci,
-                                        inc.context(),
-                                        extractBlock(inc.text(),inc.blockId()),
-                                        langExt,
-                                        inc.isExample(),
-                                        inc.exampleFile()
-                                       );
-      if (!m_firstCol) m_t << "\n";
+      CodeFragmentManager::instance().parseCodeFragment(m_ci,
+                                          inc.file(),
+                                          inc.blockId(),
+                                          inc.context(),
+                                          inc.type()==DocInclude::SnippetWithLines,
+                                          inc.trimLeft(),
+                                          inc.stripCodeComments()
+                                         );
       m_t << ".fi\n";
       m_t << ".PP\n";
       m_firstCol=TRUE;
       break;
-    case DocInclude::SnipWithLines:
-      {
-         if (!m_firstCol) m_t << "\n";
-         m_t << ".PP\n";
-         m_t << ".nf\n";
-         FileInfo cfi( inc.file().str() );
-         FileDef *fd = createFileDef( cfi.dirPath(), cfi.fileName() );
-         getCodeParser(inc.extension()).parseCode(m_ci,
-                                           inc.context(),
-                                           extractBlock(inc.text(),inc.blockId()),
-                                           langExt,
-                                           inc.isExample(),
-                                           inc.exampleFile(),
-                                           fd,
-                                           lineBlock(inc.text(),inc.blockId()),
-                                           -1,    // endLine
-                                           FALSE, // inlineFragment
-                                           0,     // memberDef
-                                           TRUE   // show line number
-                                          );
-         delete fd;
-         if (!m_firstCol) m_t << "\n";
-         m_t << ".fi\n";
-         m_t << ".PP\n";
-         m_firstCol=TRUE;
-      }
-      break;
-    case DocInclude::SnippetDoc:
-    case DocInclude::IncludeDoc:
-      err("Internal inconsistency: found switch SnippetDoc / IncludeDoc in file: %s"
-          "Please create a bug report\n",__FILE__);
-      break;
   }
 }
 
@@ -401,7 +370,7 @@ void ManDocVisitor::operator()(const DocIncOperator &op)
     m_hide = popHidden();
     if (!m_hide)
     {
-      FileDef *fd = 0;
+      std::unique_ptr<FileDef> fd;
       if (!op.includeFileName().isEmpty())
       {
         FileInfo cfi( op.includeFileName().str() );
@@ -409,15 +378,15 @@ void ManDocVisitor::operator()(const DocIncOperator &op)
       }
 
       getCodeParser(locLangExt).parseCode(m_ci,op.context(),op.text(),langExt,
+                                        op.stripCodeComments(),
                                         op.isExample(),op.exampleFile(),
-                                        fd,     // fileDef
+                                        fd.get(),     // fileDef
                                         op.line(),    // startLine
                                         -1,    // endLine
                                         FALSE, // inline fragment
-                                        0,     // memberDef
+                                        nullptr,     // memberDef
                                         op.showLineNo()  // show line numbers
                                        );
-      if (fd) delete fd;
     }
     pushHidden(m_hide);
     m_hide=TRUE;
@@ -477,9 +446,9 @@ void ManDocVisitor::operator()(const DocSeparator &s)
 void ManDocVisitor::operator()(const DocAutoList &l)
 {
   if (m_hide) return;
-  m_indent+=2;
+  m_indent++;
   visitChildren(l);
-  m_indent-=2;
+  m_indent--;
   m_t << ".PP\n";
 }
 
@@ -487,17 +456,31 @@ void ManDocVisitor::operator()(const DocAutoListItem &li)
 {
   if (m_hide) return;
   QCString ws;
-  ws.fill(' ',m_indent-2);
+  if (m_indent>0) ws.fill(' ',2*(m_indent-1));
   if (!m_firstCol) m_t << "\n";
   m_t << ".IP \"" << ws;
   const DocAutoList *list = std::get_if<DocAutoList>(li.parent());
   if (list && list->isEnumList())
   {
-    m_t << li.itemNumber() << ".\" " << m_indent+2;
+    m_t << li.itemNumber() << ".\" " << (2*(m_indent+1));
   }
   else // bullet list
   {
-    m_t << "\\(bu\" " << m_indent;
+    switch (li.itemNumber())
+    {
+      case DocAutoList::Unchecked: // unchecked
+        m_t << "[ ]\" " << (2*m_indent) + 2;
+        break;
+      case DocAutoList::Checked_x: // checked with x
+        m_t << "[x]\" " << (2*m_indent) + 2;
+        break;
+      case DocAutoList::Checked_X: // checked with X
+        m_t << "[X]\" " << (2*m_indent) + 2;
+        break;
+      default:
+        m_t << "\\(bu\" " << (2*m_indent);
+        break;
+    }
   }
   m_t << "\n";
   m_firstCol=TRUE;
@@ -517,7 +500,7 @@ void ManDocVisitor::operator()(const DocPara &p)
      )
   {
     if (!m_firstCol) m_t << "\n";
-    m_t << ".PP\n";
+    m_t << "\n.PP\n";
     m_firstCol=TRUE;
   }
 }
@@ -568,6 +551,8 @@ void ManDocVisitor::operator()(const DocSimpleSect &s)
       m_t << theTranslator->trRemarks(); break;
     case DocSimpleSect::Attention:
       m_t << theTranslator->trAttention(); break;
+    case DocSimpleSect::Important:
+      m_t << theTranslator->trImportant(); break;
     case DocSimpleSect::User: break;
     case DocSimpleSect::Rcs: break;
     case DocSimpleSect::Unknown:  break;
@@ -596,11 +581,12 @@ void ManDocVisitor::operator()(const DocTitle &t)
 void ManDocVisitor::operator()(const DocSimpleList &l)
 {
   if (m_hide) return;
-  m_indent+=2;
+  m_indent++;
   if (!m_firstCol) m_t << "\n";
   m_t << ".PD 0\n";
+  m_firstCol=true;
   visitChildren(l);
-  m_indent-=2;
+  m_indent--;
   m_t << ".PP\n";
 }
 
@@ -608,7 +594,7 @@ void ManDocVisitor::operator()(const DocSimpleListItem &li)
 {
   if (m_hide) return;
   QCString ws;
-  ws.fill(' ',m_indent-2);
+  if (m_indent>0) ws.fill(' ',2*(m_indent-1));
   if (!m_firstCol) m_t << "\n";
   m_t << ".IP \"" << ws << "\\(bu\" " << m_indent << "\n";
   m_firstCol=TRUE;
@@ -626,7 +612,10 @@ void ManDocVisitor::operator()(const DocSection &s)
   if (!m_firstCol) m_t << "\n";
   if (s.level()==1) m_t << ".SH"; else m_t << ".SS";
   m_t << " \"";
-  filter(s.title());
+  if (s.title())
+  {
+    std::visit(*this,*s.title());
+  }
   m_t << "\"\n";
   if (s.level()==1) m_t << ".PP\n";
   m_firstCol=TRUE;
@@ -636,26 +625,28 @@ void ManDocVisitor::operator()(const DocSection &s)
 void ManDocVisitor::operator()(const DocHtmlList &l)
 {
   if (m_hide) return;
-  m_indent+=2;
+  m_indent++;
   if (!m_firstCol) m_t << "\n";
   m_t << ".PD 0\n";
-  man_listItemInfo[m_indent].number = 1;
-  man_listItemInfo[m_indent].type   = '1';
+  m_firstCol=true;
+  int indent = std::min(m_indent,maxIndentLevels-1);
+  m_listItemInfo[indent].number = 1;
+  m_listItemInfo[indent].type   = '1';
   for (const auto &opt : l.attribs())
   {
     if (opt.name=="type")
     {
-      man_listItemInfo[m_indent].type = opt.value[0];
+      m_listItemInfo[indent].type = opt.value[0];
     }
     if (opt.name=="start")
     {
-      bool ok;
+      bool ok = false;
       int val = opt.value.toInt(&ok);
-      if (ok) man_listItemInfo[m_indent].number = val;
+      if (ok) m_listItemInfo[indent].number = val;
     }
   }
   visitChildren(l);
-  m_indent-=2;
+  m_indent--;
   if (!m_firstCol) m_t << "\n";
   m_t << ".PP\n";
 }
@@ -664,48 +655,49 @@ void ManDocVisitor::operator()(const DocHtmlListItem &li)
 {
   if (m_hide) return;
   QCString ws;
-  ws.fill(' ',m_indent-2);
+  if (m_indent>0) ws.fill(' ',2*(m_indent-1));
   if (!m_firstCol) m_t << "\n";
   m_t << ".IP \"" << ws;
   const DocHtmlList *list = std::get_if<DocHtmlList>(li.parent());
   if (list && list->type()==DocHtmlList::Ordered)
   {
+    int indent = std::min(m_indent,maxIndentLevels-1);
     for (const auto &opt : li.attribs())
     {
       if (opt.name=="value")
       {
-        bool ok;
+        bool ok = false;
         int val = opt.value.toInt(&ok);
-        if (ok) man_listItemInfo[m_indent].number = val;
+        if (ok) m_listItemInfo[indent].number = val;
       }
     }
-    switch (man_listItemInfo[m_indent].type)
+    switch (m_listItemInfo[indent].type)
     {
       case '1':
-        m_t << man_listItemInfo[m_indent].number;
+        m_t << m_listItemInfo[indent].number;
         break;
       case 'a':
-        m_t << integerToAlpha(man_listItemInfo[m_indent].number,false);
+        m_t << integerToAlpha(m_listItemInfo[indent].number,false);
         break;
       case 'A':
-        m_t << integerToAlpha(man_listItemInfo[m_indent].number);
+        m_t << integerToAlpha(m_listItemInfo[indent].number);
         break;
       case 'i':
-        m_t << integerToRoman(man_listItemInfo[m_indent].number,false);
+        m_t << integerToRoman(m_listItemInfo[indent].number,false);
         break;
       case 'I':
-        m_t << integerToRoman(man_listItemInfo[m_indent].number);
+        m_t << integerToRoman(m_listItemInfo[indent].number);
         break;
       default:
-        m_t << man_listItemInfo[m_indent].number;
+        m_t << m_listItemInfo[indent].number;
         break;
     }
-    m_t << ".\" " << m_indent+2;
-    man_listItemInfo[m_indent].number++;
+    m_t << ".\" " << ((m_indent+1)*2);
+    m_listItemInfo[indent].number++;
   }
   else // bullet list
   {
-    m_t << "\\(bu\" " << m_indent;
+    m_t << "\\(bu\" " << (m_indent*2);
   }
   m_t << "\n";
   m_firstCol=TRUE;
@@ -717,9 +709,11 @@ void ManDocVisitor::operator()(const DocHtmlListItem &li)
 void ManDocVisitor::operator()(const DocHtmlDescList &dl)
 {
   if (m_hide) return;
+  m_indent+=2;
   visitChildren(dl);
+  m_indent-=2;
   if (!m_firstCol) m_t << "\n";
-  m_t << ".PP\n";
+  m_t << "\n.PP\n";
   m_firstCol=TRUE;
 }
 
@@ -727,15 +721,17 @@ void ManDocVisitor::operator()(const DocHtmlDescTitle &dt)
 {
   if (m_hide) return;
   if (!m_firstCol) m_t << "\n";
-  m_t << ".IP \"\\fB";
+  m_t << "\n.PP";
+  m_t << "\n.IP \"\\fB";
   m_firstCol=FALSE;
   visitChildren(dt);
-  m_t << "\\fP\" 1c\n";
-  m_firstCol=TRUE;
 }
 
 void ManDocVisitor::operator()(const DocHtmlDescData &dd)
 {
+  if (!m_firstCol) m_t << "\n";
+  m_t << ".IP \"\" 1c\n";
+  m_firstCol=TRUE;
   visitChildren(dd);
 }
 
@@ -767,7 +763,7 @@ void ManDocVisitor::operator()(const DocInternal &i)
 void ManDocVisitor::operator()(const DocHRef &href)
 {
   if (m_hide) return;
-  m_t << "\\fC";
+  m_t << "\\fR";
   visitChildren(href);
   m_t << "\\fP";
 }
@@ -820,7 +816,7 @@ void ManDocVisitor::operator()(const DocImage &)
 {
 }
 
-void ManDocVisitor::operator()(const DocDotFile &f)
+void ManDocVisitor::operator()(const DocDotFile &)
 {
 }
 
@@ -832,6 +828,10 @@ void ManDocVisitor::operator()(const DocDiaFile &)
 {
 }
 
+void ManDocVisitor::operator()(const DocPlantUmlFile &)
+{
+}
+
 void ManDocVisitor::operator()(const DocLink &dl)
 {
   if (m_hide) return;
@@ -853,9 +853,9 @@ void ManDocVisitor::operator()(const DocSecRefItem &ref)
 {
   if (m_hide) return;
   QCString ws;
-  ws.fill(' ',m_indent-2);
+  if (m_indent>0) ws.fill(' ',2*(m_indent-1));
   if (!m_firstCol) m_t << "\n";
-  m_t << ".IP \"" << ws << "\\(bu\" " << m_indent << "\n";
+  m_t << ".IP \"" << ws << "\\(bu\" " << (2*m_indent) << "\n";
   m_firstCol=TRUE;
   visitChildren(ref);
   m_t << "\n";
@@ -865,9 +865,9 @@ void ManDocVisitor::operator()(const DocSecRefItem &ref)
 void ManDocVisitor::operator()(const DocSecRefList &l)
 {
   if (m_hide) return;
-  m_indent+=2;
+  m_indent++;
   visitChildren(l);
-  m_indent-=2;
+  m_indent--;
   if (!m_firstCol) m_t << "\n";
   m_t << ".PP\n";
 }
@@ -996,7 +996,6 @@ void ManDocVisitor::filter(const QCString &str)
       {
         case '.':  m_t << "\\&."; break; // see  bug652277
         case '\\': m_t << "\\\\"; break;
-        case '"':  c = '\''; // fall through
         default: m_t << c; break;
       }
     }
