@@ -1,8 +1,6 @@
 /******************************************************************************
  *
- *
- *
- * Copyright (C) 1997-2015 by Dimitri van Heesch.
+ * Copyright (C) 1997-2022 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby
@@ -19,18 +17,21 @@
    nice introductions to groff and man pages. */
 
 #include <stdlib.h>
+#include <string.h>
 
 #include "message.h"
 #include "mangen.h"
 #include "config.h"
 #include "util.h"
 #include "doxygen.h"
-#include <string.h>
 #include "docparser.h"
 #include "mandocvisitor.h"
 #include "language.h"
 #include "dir.h"
 #include "utf8.h"
+#include "datetime.h"
+#include "portable.h"
+#include "outputlist.h"
 
 static QCString getExtension()
 {
@@ -75,27 +76,197 @@ static QCString getSubdir()
   return dir;
 }
 
-ManGenerator::ManGenerator() : OutputGenerator(Config_getString(MAN_OUTPUT)+"/"+getSubdir())
+static QCString docifyToString(const QCString &str)
+{
+  QCString result;
+  result.reserve(str.length());
+  if (!str.isEmpty())
+  {
+    const char *p=str.data();
+    char c=0;
+    while ((c=*p++))
+    {
+      switch(c)
+      {
+        case '-':  result += "\\-";  break; // see  bug747780
+        case '.':  result += "\\&."; break; // see  bug652277
+        case '\\': result += "\\\\"; break;
+        case '\n': result += "\n";   break;
+        case '\"': c = '\'';   // no break!
+        default:   result += c;      break;
+      }
+    }
+    //printf("%s",str);fflush(stdout);
+  }
+  return result;
+}
+
+static QCString objectLinkToString(const QCString &text)
+{
+  return "\\fB" + docifyToString(text) + "\\fP";
+}
+
+//-------------------------------------------------------------------------------
+
+ManCodeGenerator::ManCodeGenerator(TextStream *t) : m_t(t)
 {
 }
 
-ManGenerator::ManGenerator(const ManGenerator &og) : OutputGenerator(og)
+void ManCodeGenerator::startCodeFragment(const QCString &)
 {
+  *m_t << "\n";
+  *m_t << ".nf\n";
+}
+
+void ManCodeGenerator::endCodeFragment(const QCString &)
+{
+  if (m_col>0) *m_t << "\n";
+  *m_t << ".PP\n";
+  *m_t << ".fi\n";
+  m_col=0;
+}
+
+void ManCodeGenerator::writeLineNumber(const QCString &,const QCString &,const QCString &,int l, bool)
+{
+  if (m_hide) return;
+  *m_t << l << " ";
+  m_col=0;
+}
+
+void ManCodeGenerator::writeCodeLink(CodeSymbolType,
+                                 const QCString &,const QCString &,
+                                 const QCString &, const QCString &name,
+                                 const QCString &)
+{
+  if (m_hide) return;
+  if (!name.isEmpty())
+  {
+    const char *p=name.data();
+    char c=0;
+    while ((c=*p++))
+    {
+      switch(c)
+      {
+        case '-':  *m_t << "\\-"; break; // see  bug747780
+        case '.':  *m_t << "\\&."; break; // see  bug652277
+        case '\\': *m_t << "\\\\"; m_col++; break;
+        case '\n': *m_t << "\n"; m_col=0; break;
+        case '\"':  c = '\''; // no break!
+        default: *m_t << c; m_col++; break;
+      }
+    }
+    //printf("%s",str);fflush(stdout);
+  }
+}
+
+void ManCodeGenerator::codify(const QCString &str)
+{
+  const int tabSize = Config_getInt(TAB_SIZE);
+  const size_t stripAmount = m_stripIndentAmount;
+  if (!str.isEmpty())
+  {
+    char c;
+    const char *p=str.data();
+    if (m_hide)
+    {
+      m_col = updateColumnCount(p,m_col);
+    }
+    else
+    {
+      while ((c=*p++))
+      {
+        switch(c)
+        {
+          case '-':  *m_t << "\\-"; break; // see  bug747780
+          case '.':   *m_t << "\\&."; break; // see  bug652277
+          case '\t':  {
+                        int spacesToNextTabStop = tabSize - (m_col%tabSize);
+                        while (spacesToNextTabStop--)
+                        {
+                          if (m_col>=stripAmount) *m_t << " ";
+                          m_col++;
+                        }
+                      }
+                      break;
+          case ' ':   if (m_col>=stripAmount) *m_t << " ";
+                      m_col++;
+                      break;
+          case '\n':  *m_t << "\n"; m_col=0; break;
+          case '\\':  *m_t << "\\\\"; m_col++; break;
+          case '\"':  // no break!
+          default:    p=writeUTF8Char(*m_t,p-1); m_col++; break;
+        }
+      }
+      //printf("%s",str);fflush(stdout);
+    }
+  }
+}
+
+void ManCodeGenerator::stripCodeComments(bool b)
+{
+  m_stripCodeComments = b;
+}
+
+void ManCodeGenerator::startSpecialComment()
+{
+  m_hide = m_stripCodeComments;
+}
+
+void ManCodeGenerator::endSpecialComment()
+{
+  m_hide = false;
+}
+
+void ManCodeGenerator::setStripIndentAmount(size_t amount)
+{
+  m_stripIndentAmount = amount;
+}
+
+//-------------------------------------------------------------------------------
+
+ManGenerator::ManGenerator()
+  : OutputGenerator(Config_getString(MAN_OUTPUT)+"/"+getSubdir())
+  , m_codeList(std::make_unique<OutputCodeList>())
+{
+  m_codeGen = m_codeList->add<ManCodeGenerator>(&m_t);
+}
+
+ManGenerator::ManGenerator(const ManGenerator &og) : OutputGenerator(og.m_dir), OutputGenIntf()
+{
+  m_codeList = std::make_unique<OutputCodeList>(*og.m_codeList);
+  m_codeGen      = m_codeList->get<ManCodeGenerator>(OutputType::Man);
+  m_codeGen->setTextStream(&m_t);
+  m_firstCol      = og.m_firstCol;
+  m_col           = og.m_col;
+  m_paragraph     = og.m_paragraph;
+  m_upperCase     = og.m_upperCase;
+  m_insideTabbing = og.m_insideTabbing;
+  m_inHeader      = og.m_inHeader;
 }
 
 ManGenerator &ManGenerator::operator=(const ManGenerator &og)
 {
-  OutputGenerator::operator=(og);
+  if (this!=&og)
+  {
+    m_dir           = og.m_dir;
+    m_codeList = std::make_unique<OutputCodeList>(*og.m_codeList);
+    m_codeGen       = m_codeList->get<ManCodeGenerator>(OutputType::Man);
+    m_codeGen->setTextStream(&m_t);
+    m_firstCol      = og.m_firstCol;
+    m_col           = og.m_col;
+    m_paragraph     = og.m_paragraph;
+    m_upperCase     = og.m_upperCase;
+    m_insideTabbing = og.m_insideTabbing;
+    m_inHeader      = og.m_inHeader;
+  }
   return *this;
 }
 
-std::unique_ptr<OutputGenerator> ManGenerator::clone() const
-{
-  return std::make_unique<ManGenerator>(*this);
-}
+ManGenerator::~ManGenerator() = default;
 
-ManGenerator::~ManGenerator()
+void ManGenerator::addCodeGen(OutputCodeList &list)
 {
+  list.add<ManCodeGeneratorDefer>(m_codeGen);
 }
 
 void ManGenerator::init()
@@ -128,7 +299,7 @@ static QCString buildFileName(const QCString &name)
   if (name.isEmpty()) return "noname";
 
   const char *p=name.data();
-  char c;
+  char c = 0;
   while ((c=*p++))
   {
     switch (c)
@@ -163,7 +334,7 @@ static QCString buildFileName(const QCString &name)
   return fileName;
 }
 
-void ManGenerator::startFile(const QCString &,const QCString &manName,const QCString &,int)
+void ManGenerator::startFile(const QCString &,const QCString &manName,const QCString &,int,int)
 {
   startPlainFile( buildFileName( manName ) );
   m_firstCol=TRUE;
@@ -177,8 +348,19 @@ void ManGenerator::endFile()
 
 void ManGenerator::endTitleHead(const QCString &,const QCString &name)
 {
-  m_t << ".TH \"" << name << "\" " << getExtension() << " \""
-    << dateToString(FALSE) << "\" \"";
+  m_t << ".TH \"" << name << "\" " << getExtension() << " \"";
+  switch (Config_getEnum(TIMESTAMP))
+  {
+    case TIMESTAMP_t::YES:
+    case TIMESTAMP_t::DATETIME:
+      m_t << dateToString(DateTimeType::DateTime) << "\" \"";
+      break;
+    case TIMESTAMP_t::DATE:
+      m_t << dateToString(DateTimeType::Date) << "\" \"";
+      break;
+    case TIMESTAMP_t::NO:
+      break;
+  }
   if (!Config_getString(PROJECT_NUMBER).isEmpty())
     m_t << "Version " << Config_getString(PROJECT_NUMBER) << "\" \"";
   if (Config_getString(PROJECT_NAME).isEmpty())
@@ -240,27 +422,11 @@ void ManGenerator::writeStartAnnoItem(const QCString &,const QCString &,
 }
 
 void ManGenerator::writeObjectLink(const QCString &,const QCString &,
-                                    const QCString &, const QCString &name)
+                                   const QCString &, const QCString &name)
 {
   startBold(); docify(name); endBold();
 }
 
-void ManGenerator::writeCodeLink(CodeSymbolType,
-                                 const QCString &,const QCString &,
-                                 const QCString &, const QCString &name,
-                                 const QCString &)
-{
-  docify(name);
-}
-
-void ManGenerator::startHtmlLink(const QCString &)
-{
-}
-
-void ManGenerator::endHtmlLink()
-{
-}
-
 void ManGenerator::startGroupHeader(int)
 {
   if (!m_firstCol) m_t << "\n";
@@ -300,12 +466,12 @@ void ManGenerator::docify(const QCString &str)
     {
       switch(c)
       {
-        case '-':  m_t << "\\-"; break; // see  bug747780
-        case '.':  m_t << "\\&."; break; // see  bug652277
+        case '-':  m_t << "\\-";           break; // see  bug747780
+        case '.':  m_t << "\\&.";          break; // see  bug652277
         case '\\': m_t << "\\\\"; m_col++; break;
-        case '\n': m_t << "\n"; m_col=0; break;
-        case '\"':  c = '\''; // no break!
-        default: m_t << c; m_col++; break;
+        case '\n': m_t << "\n";   m_col=0; break;
+        case '\"': c = '\'';         // no break!
+        default:   m_t << c;      m_col++; break;
       }
     }
     m_firstCol=(c=='\n');
@@ -314,36 +480,6 @@ void ManGenerator::docify(const QCString &str)
   m_paragraph=FALSE;
 }
 
-void ManGenerator::codify(const QCString &str)
-{
-  //static char spaces[]="        ";
-  if (!str.isEmpty())
-  {
-    const char *p=str.data();
-    char c;
-    int spacesToNextTabStop;
-    while (*p)
-    {
-      c=*p++;
-      switch(c)
-      {
-        case '.':   m_t << "\\&."; break; // see  bug652277
-        case '\t':  spacesToNextTabStop =
-                          Config_getInt(TAB_SIZE) - (m_col%Config_getInt(TAB_SIZE));
-                    m_t << Doxygen::spaces.left(spacesToNextTabStop);
-                    m_col+=spacesToNextTabStop;
-                    break;
-        case '\n':  m_t << "\n"; m_firstCol=TRUE; m_col=0; break;
-        case '\\':  m_t << "\\"; m_col++; break;
-        case '\"':  // no break!
-        default:    p=writeUTF8Char(m_t,p-1); m_firstCol=FALSE; m_col++; break;
-      }
-    }
-    //printf("%s",str);fflush(stdout);
-  }
-  m_paragraph=FALSE;
-}
-
 void ManGenerator::writeChar(char c)
 {
   m_firstCol=(c=='\n');
@@ -358,17 +494,6 @@ void ManGenerator::writeChar(char c)
   m_paragraph=FALSE;
 }
 
-void ManGenerator::startDescList(SectionTypes)
-{
-  if (!m_firstCol)
-  { m_t << "\n" << ".PP\n";
-    m_firstCol=TRUE; m_paragraph=TRUE;
-    m_col=0;
-  }
-  m_paragraph=FALSE;
-  startBold();
-}
-
 void ManGenerator::startTitle()
 {
   if (!m_firstCol) m_t << "\n";
@@ -395,23 +520,6 @@ void ManGenerator::endItemListItem()
 {
 }
 
-void ManGenerator::startCodeFragment(const QCString &)
-{
-  newParagraph();
-  m_t << ".nf\n";
-  m_firstCol=TRUE;
-  m_paragraph=FALSE;
-}
-
-void ManGenerator::endCodeFragment(const QCString &)
-{
-  if (!m_firstCol) m_t << "\n";
-  m_t << ".fi\n";
-  m_firstCol=TRUE;
-  m_paragraph=FALSE;
-  m_col=0;
-}
-
 void ManGenerator::startMemberDoc(const QCString &,const QCString &,const QCString &,const QCString &,int,int,bool)
 {
   if (!m_firstCol) m_t << "\n";
@@ -444,7 +552,7 @@ void ManGenerator::startDoxyAnchor(const QCString &,const QCString &manName,
     FileInfo fi(fileName.str());
     if (!fi.exists())
     {
-      std::ofstream linkStream(fileName.str(),std::ofstream::out | std::ofstream::binary);
+      std::ofstream linkStream = Portable::openOutputStream(fileName);
       if (linkStream.is_open())
       {
         linkStream << ".so " << getSubdir() << "/" << buildFileName( manName ) << "\n";
@@ -452,26 +560,16 @@ void ManGenerator::startDoxyAnchor(const QCString &,const QCString &manName,
     }
 }
 
+void ManGenerator::addLabel(const QCString &,const QCString &)
+{
+}
+
 void ManGenerator::endMemberDoc(bool)
 {
     m_t << "\"\n";
 }
 
-void ManGenerator::startSubsection()
-{
-  if (!m_firstCol) m_t << "\n";
-  m_t << ".SS \"";
-  m_firstCol=FALSE;
-  m_paragraph=FALSE;
-}
-
-void ManGenerator::endSubsection()
-{
-  m_t << "\"";
-}
-
-
-void ManGenerator::startSubsubsection()
+void ManGenerator::startCompoundTemplateParams()
 {
   if (!m_firstCol) m_t << "\n";
   m_t << "\n.SS \"";
@@ -479,11 +577,10 @@ void ManGenerator::startSubsubsection()
   m_paragraph=FALSE;
 }
 
-void ManGenerator::endSubsubsection()
+void ManGenerator::endCompoundTemplateParams()
 {
   m_t << "\"";
 }
-
 void ManGenerator::writeSynopsis()
 {
   if (!m_firstCol) m_t << "\n";
@@ -492,13 +589,6 @@ void ManGenerator::writeSynopsis()
   m_paragraph=FALSE;
 }
 
-void ManGenerator::startDescItem()
-{
-  if (!m_firstCol) m_t << "\n";
-  m_t << ".IP \"";
-  m_firstCol=FALSE;
-}
-
 void ManGenerator::startDescForItem()
 {
   if (!m_firstCol) m_t << "\n";
@@ -513,12 +603,6 @@ void ManGenerator::endDescForItem()
 {
 }
 
-void ManGenerator::endDescItem()
-{
-  m_t << "\" 1c\n";
-  m_firstCol=TRUE;
-}
-
 void ManGenerator::startAnonTypeScope(int indentLevel)
 {
   if (indentLevel==0)
@@ -536,14 +620,14 @@ void ManGenerator::endAnonTypeScope(int indentLevel)
 }
 
 
-void ManGenerator::startMemberItem(const QCString &,int,const QCString &)
+void ManGenerator::startMemberItem(const QCString &,MemberItemType,const QCString &)
 {
   if (m_firstCol && !m_insideTabbing) m_t << ".in +1c\n";
   m_t << "\n.ti -1c\n.RI \"";
   m_firstCol=FALSE;
 }
 
-void ManGenerator::endMemberItem()
+void ManGenerator::endMemberItem(MemberItemType)
 {
   m_t << "\"\n.br";
 }
@@ -599,13 +683,15 @@ void ManGenerator::startSection(const QCString &,const QCString &,SectionType ty
 {
   if( !m_inHeader )
   {
-    switch(type)
+    switch(type.level())
     {
-      case SectionType::Page:          startGroupHeader(FALSE); break;
-      case SectionType::Section:       startGroupHeader(FALSE); break;
-      case SectionType::Subsection:    startMemberHeader(QCString(), -1); break;
-      case SectionType::Subsubsection: startMemberHeader(QCString(), -1); break;
-      case SectionType::Paragraph:     startMemberHeader(QCString(), -1); break;
+      case SectionType::Page:             // fall through
+      case SectionType::Section:          startGroupHeader(0); break;
+      case SectionType::Subsection:       // fall through
+      case SectionType::Subsubsection:    // fall through
+      case SectionType::Paragraph:        // fall through
+      case SectionType::Subparagraph:     // fall through
+      case SectionType::Subsubparagraph:  startMemberHeader(QCString(), -1); break;
       default: ASSERT(0); break;
     }
   }
@@ -615,19 +701,21 @@ void ManGenerator::endSection(const QCString &,SectionType type)
 {
   if( !m_inHeader )
   {
-    switch(type)
+    switch(type.level())
     {
-      case SectionType::Page:          endGroupHeader(0); break;
-      case SectionType::Section:       endGroupHeader(0); break;
-      case SectionType::Subsection:    endMemberHeader(); break;
-      case SectionType::Subsubsection: endMemberHeader(); break;
-      case SectionType::Paragraph:     endMemberHeader(); break;
+      case SectionType::Page:            // fall through
+      case SectionType::Section:         endGroupHeader(0); break;
+      case SectionType::Subsection:      // fall through
+      case SectionType::Subsubsection:   // fall through
+      case SectionType::Paragraph:       // fall through
+      case SectionType::Subparagraph:    // fall through
+      case SectionType::Subsubparagraph: endMemberHeader(); break;
       default: ASSERT(0); break;
     }
   }
   else
   {
-    m_t << "\n";
+    m_t << "\n.PP\n";
     m_firstCol=TRUE;
     m_paragraph=FALSE;
     m_inHeader=FALSE;
@@ -652,7 +740,7 @@ void ManGenerator::endExamples()
 {
 }
 
-void ManGenerator::startDescTable(const QCString &title)
+void ManGenerator::startDescTable(const QCString &title,const bool hasInits)
 {
   if (!m_firstCol)
   { m_t << "\n.PP\n";
@@ -672,29 +760,14 @@ void ManGenerator::endDescTable()
   endDescForItem();
 }
 
-void ManGenerator::startParamList(ParamListTypes,const QCString &title)
+void ManGenerator::writeDoc(const IDocNodeAST *ast,const Definition *ctx,const MemberDef *,int)
 {
-  if (!m_firstCol)
-  { m_t << "\n.PP\n";
-    m_firstCol=TRUE; m_paragraph=TRUE;
-    m_col=0;
+  const DocNodeAST *astImpl = dynamic_cast<const DocNodeAST *>(ast);
+  if (astImpl)
+  {
+    ManDocVisitor visitor(m_t,*m_codeList,ctx?ctx->getDefFileExtension():QCString(""));
+    std::visit(visitor,astImpl->root);
   }
-  m_paragraph=FALSE;
-  startBold();
-  docify(title);
-  endBold();
-  m_paragraph=TRUE;
-}
-
-void ManGenerator::endParamList()
-{
-}
-
-void ManGenerator::writeDoc(DocNode *n,const Definition *ctx,const MemberDef *,int)
-{
-  ManDocVisitor *visitor = new ManDocVisitor(m_t,*this,ctx?ctx->getDefFileExtension():QCString(""));
-  n->accept(visitor);
-  delete visitor;
   m_firstCol=FALSE;
   m_paragraph = FALSE;
 }
@@ -829,7 +902,7 @@ void ManGenerator::startLabels()
 
 void ManGenerator::writeLabel(const QCString &l,bool isLast)
 {
-  m_t << "\\fC [" << l << "]\\fP";
+  m_t << "\\fR [" << l << "]\\fP";
   if (!isLast) m_t << ", ";
 }
 
@@ -840,3 +913,31 @@ void ManGenerator::endLabels()
 void ManGenerator::endHeaderSection()
 {
 }
+
+void ManGenerator::writeInheritedSectionTitle(
+                  const QCString &/*id*/,    const QCString &/*ref*/,
+                  const QCString &/*file*/,  const QCString &/*anchor*/,
+                  const QCString &title,     const QCString &name)
+{
+  m_t << "\n\n";
+  m_t << theTranslator->trInheritedFrom(docifyToString(title), objectLinkToString(name));
+  m_firstCol = FALSE;
+}
+
+void ManGenerator::startParameterList(bool openBracket)
+{
+  if (openBracket) m_t << "(";
+}
+
+void ManGenerator::endParameterExtra(bool last,bool /* emptyList */, bool closeBracket)
+{
+  if (last && closeBracket)
+  {
+    m_t << ")";
+  }
+}
+void ManGenerator::endParameterType()
+{
+  m_t << " ";
+}
+
