@@ -59,8 +59,12 @@ class ModuleDefImpl : public DefinitionMixin<ModuleDef>
     QCString displayName(bool=TRUE) const override { return name(); }
     QCString getOutputFileBase() const override;
     QCString anchor() const override { return ""; }
-    bool isLinkableInProject() const override { return isLinkable() && !isHidden() && !isReference(); }
-    bool isLinkable() const override { return hasDocumentation(); }
+    bool isLinkableInProject() const override {
+      if (m_primaryInterface) return m_primaryInterface->isLinkableInProject();
+      else return isLinkable() && !isHidden() && !isReference(); }
+    bool isLinkable() const override {
+      if (m_primaryInterface) return m_primaryInterface->isLinkable();
+      else return hasDocumentation(); }
     QCString qualifiedName() const override;
     void writeSummaryLinks(OutputList &ol) const override;
 
@@ -398,7 +402,7 @@ void ModuleDefImpl::writeDocumentation(OutputList &ol)
       case LayoutDocEntry::MemberDecl:
         {
           const LayoutDocEntryMemberDecl *lmd = dynamic_cast<const LayoutDocEntryMemberDecl*>(lde.get());
-          if (lmd && lmd->visible()) writeMemberDeclarations(ol,lmd->type,lmd->title(lang));
+          if (lmd) writeMemberDeclarations(ol,lmd->type,lmd->title(lang));
         }
         break;
       case LayoutDocEntry::MemberDeclEnd:
@@ -413,7 +417,7 @@ void ModuleDefImpl::writeDocumentation(OutputList &ol)
       case LayoutDocEntry::MemberDef:
         {
           const LayoutDocEntryMemberDef *lmd = dynamic_cast<const LayoutDocEntryMemberDef*>(lde.get());
-          if (lmd && lmd->visible()) writeMemberDocumentation(ol,lmd->type,lmd->title(lang));
+          if (lmd) writeMemberDocumentation(ol,lmd->type,lmd->title(lang));
         }
         break;
       case LayoutDocEntry::MemberDefEnd:
@@ -800,7 +804,7 @@ void ModuleDefImpl::writeSummaryLinks(OutputList &ol) const
     else if (lde->kind()==LayoutDocEntry::MemberDecl)
     {
       const LayoutDocEntryMemberDecl *lmd = dynamic_cast<const LayoutDocEntryMemberDecl*>(lde.get());
-      if (lmd && lmd->visible())
+      if (lmd)
       {
         MemberList * ml = getMemberList(lmd->type);
         if (ml && ml->declVisible())
@@ -987,7 +991,7 @@ int ModuleDefImpl::countVisibleMembers() const
     if (lde->kind()==LayoutDocEntry::MemberDecl)
     {
       const LayoutDocEntryMemberDecl *lmd = dynamic_cast<const LayoutDocEntryMemberDecl*>(lde.get());
-      if (lmd && lmd->visible())
+      if (lmd)
       {
         MemberList *ml = getMemberList(lmd->type);
         if (ml)
@@ -1508,6 +1512,11 @@ const ModuleLinkedMap &ModuleManager::modules() const
   return p->moduleFileMap;
 }
 
+ModuleLinkedMap &ModuleManager::modules()
+{
+  return p->moduleFileMap;
+}
+
 void ModuleManager::addDocs(const Entry *root)
 {
   AUTO_TRACE("file={} module={}",root->fileName,root->name);
