@@ -1,6 +1,5 @@
 /*
- * Copyright (C) 2008 by Sebastian Pipping.
- * Copyright (C) 2008 Dimitri van Heesch.
+ * Copyright (C) 1997-2022 Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby
@@ -10,23 +9,27 @@
  *
  * Documents produced by Doxygen are derivative works derived from the
  * input used in their production; they are not affected by this license.
- *
- * Sebastian Pipping <sebastian@pipping.org>
  */
 
-#ifndef DOXYGEN_QHP_H
-#define DOXYGEN_QHP_H
+#ifndef QHP_H
+#define QHP_H
 
-#include "index.h"
-#include "qhpxmlwriter.h"
+#include <memory>
+
+#include "qcstring.h"
+#include "construct.h"
+#include "indexlist.h"
+
+class Definition;
+class MemberDef;
 
 class Qhp : public IndexIntf
 {
   public:
     Qhp();
-   ~Qhp();
+    ~Qhp();
+    NON_COPYABLE(Qhp)
 
-    // BEGIN IndexIntf
     void initialize();
     void finalize();
     void incContentsDepth();
@@ -40,32 +43,15 @@ class Qhp : public IndexIntf
     void addIndexFile(const QCString & name);
     void addImageFile(const QCString & name);
     void addStyleSheetFile(const QCString & name);
-    // END IndexIntf
 
-    static QCString getQhpFileName();
+    static inline const QCString qhpFileName = "index.qhp";
+    static QCString getQchFileName();
 
   private:
-    void handlePrevSection();
-    void clearPrevSection();
-    void setPrevSection(const QCString & title, const QCString & basename, const QCString & anchor, int level);
-    void addFile(const QCString & fileName);
-
-    static QCString getFullProjectName();
-
-    QhpXmlWriter m_doc;
-    QhpXmlWriter m_toc;
-    QhpXmlWriter m_index;
-    QhpXmlWriter m_files;
-
-    QCString m_prevSectionTitle;
-    QCString m_prevSectionBaseName;
-    QCString m_prevSectionAnchor;
-
-    int m_prevSectionLevel = 0;
-    int m_sectionLevel = 0;
-    int m_openCount = 0;
-    bool m_skipMainPageSection = false;
+    void addFile(const QCString &);
+    class Private;
+    std::unique_ptr<Private> p;
 };
 
-#endif // DOXYGEN_QHP_H
+#endif // QHP_H
 
