@@ -16,7 +16,6 @@
 
 #include <ctype.h>
 #include <assert.h>
-#include <sstream>
 #include <mutex>
 #include <map>
 #include <unordered_map>
@@ -32,6 +31,7 @@
 #include "message.h"
 #include "groupdef.h"
 #include "filedef.h"
+#include "portable.h"
 
 
 // file format: (all multi-byte values are stored in big endian format)
@@ -47,6 +47,8 @@
 
 const size_t numIndexEntries = 256*256;
 
+static std::mutex g_searchIndexMutex;
+
 //--------------------------------------------------------------------
 
 void SearchIndex::IndexWord::addUrlIndex(int idx,bool hiPriority)
@@ -56,7 +58,7 @@ void SearchIndex::IndexWord::addUrlIndex(int idx,bool hiPriority)
   if (it==m_urls.end())
   {
     //printf("URLInfo::URLInfo(%d)\n",idx);
-    it = m_urls.insert(std::make_pair(idx,URLInfo(idx,0))).first;
+    it = m_urls.emplace(idx,URLInfo(idx,0)).first;
   }
   it->second.freq+=2;
   if (hiPriority) it->second.freq|=1; // mark as high priority document
@@ -71,7 +73,8 @@ SearchIndex::SearchIndex()
 
 void SearchIndex::setCurrentDoc(const Definition *ctx,const QCString &anchor,bool isSourceFile)
 {
-  if (ctx==0) return;
+  if (ctx==nullptr) return;
+  std::lock_guard<std::mutex> lock(g_searchIndexMutex);
   assert(!isSourceFile || ctx->definitionType()==Definition::TypeFile);
   //printf("SearchIndex::setCurrentDoc(%s,%s,%s)\n",name,baseName,anchor);
   QCString url=isSourceFile ? (toFileDef(ctx))->getSourceFileBase() : ctx->getOutputFileBase();
@@ -83,7 +86,7 @@ void SearchIndex::setCurrentDoc(const Definition *ctx,const QCString &anchor,boo
   if (ctx->definitionType()==Definition::TypeMember)
   {
     const MemberDef *md = toMemberDef(ctx);
-    name.prepend((md->getLanguage()==SrcLangExt_Fortran  ?
+    name.prepend((md->getLanguage()==SrcLangExt::Fortran  ?
                  theTranslator->trSubprogram(TRUE,TRUE) :
                  theTranslator->trMember(TRUE,TRUE))+" ");
   }
@@ -118,11 +121,11 @@ void SearchIndex::setCurrentDoc(const Definition *ctx,const QCString &anchor,boo
         break;
       case Definition::TypeNamespace:
         {
-          if (lang==SrcLangExt_Java || lang==SrcLangExt_CSharp)
+          if (lang==SrcLangExt::Java || lang==SrcLangExt::CSharp)
           {
             name = theTranslator->trPackage(name);
           }
-          else if (lang==SrcLangExt_Fortran)
+          else if (lang==SrcLangExt::Fortran)
           {
             name.prepend(theTranslator->trModule(TRUE,TRUE)+" ");
           }
@@ -145,21 +148,27 @@ void SearchIndex::setCurrentDoc(const Definition *ctx,const QCString &anchor,boo
           }
         }
         break;
+      case Definition::TypeModule:
+        {
+          name.prepend(theTranslator->trModule(TRUE,TRUE)+" ");
+        }
+        break;
       default:
         break;
     }
   }
 
   auto it = m_url2IdMap.find(baseUrl.str());
-  if (it == m_url2IdMap.end())
+  if (it == m_url2IdMap.end()) // new entry
   {
-    ++m_urlIndex;
-    m_url2IdMap.insert(std::make_pair(baseUrl.str(),m_urlIndex));
-    m_urls.insert(std::make_pair(m_urlIndex,URL(name,url)));
+    m_urlIndex = m_urlMaxIndex++;
+    m_url2IdMap.emplace(baseUrl.str(),m_urlIndex);
+    m_urls.emplace(m_urlIndex,URL(name,url));
   }
-  else
+  else // existing entry
   {
-    m_urls.insert(std::make_pair(it->second,URL(name,url)));
+    m_urlIndex=it->second;
+    m_urls.emplace(it->second,URL(name,url));
   }
 }
 
@@ -168,9 +177,9 @@ static int charsToIndex(const QCString &word)
   if (word.length()<2) return -1;
 
   // Fast string hashing algorithm
-  //register ushort h=0;
+  //register uint16_t h=0;
   //const char *k = word;
-  //ushort mask=0xfc00;
+  //uint16_t mask=0xfc00;
   //while ( *k )
   //{
   //  h = (h&mask)^(h<<6)^(*k++);
@@ -178,8 +187,8 @@ static int charsToIndex(const QCString &word)
   //return h;
 
   // Simple hashing that allows for substring searching
-  uint c1=static_cast<uchar>(word[0]);
-  uint c2=static_cast<uchar>(word[1]);
+  uint32_t c1=static_cast<uint8_t>(word[0]);
+  uint32_t c2=static_cast<uint8_t>(word[1]);
   return c1*256+c2;
 }
 
@@ -194,15 +203,14 @@ void SearchIndex::addWordRec(const QCString &word,bool hiPriority,bool recurse)
   if (it==m_words.end())
   {
     //fprintf(stderr,"addWord(%s) at index %d\n",word,idx);
-    m_index[idx].push_back(IndexWord(wStr));
-    it = m_words.insert({ wStr.str(), static_cast<int>(m_index[idx].size())-1 }).first;
+    m_index[idx].emplace_back(wStr);
+    it = m_words.emplace( wStr.str(), static_cast<int>(m_index[idx].size())-1 ).first;
   }
   m_index[idx][it->second].addUrlIndex(m_urlIndex,hiPriority);
-  int i;
   bool found=FALSE;
   if (!recurse) // the first time we check if we can strip the prefix
   {
-    i=getPrefixIndex(word);
+    int i=getPrefixIndex(word);
     if (i>0)
     {
       addWordRec(word.data()+i,hiPriority,TRUE);
@@ -211,7 +219,7 @@ void SearchIndex::addWordRec(const QCString &word,bool hiPriority,bool recurse)
   }
   if (!found) // no prefix stripped
   {
-    i=0;
+    int i=0;
     while (word[i]!=0 &&
            !((word[i]=='_' || word[i]==':' || (word[i]>='a' && word[i]<='z')) &&  // [_a-z:]
              (word[i+1]>='A' && word[i+1]<='Z')))                                 // [A-Z]
@@ -227,12 +235,13 @@ void SearchIndex::addWordRec(const QCString &word,bool hiPriority,bool recurse)
 
 void SearchIndex::addWord(const QCString &word,bool hiPriority)
 {
+  std::lock_guard<std::mutex> lock(g_searchIndexMutex);
   addWordRec(word,hiPriority,FALSE);
 }
 
 static void writeInt(std::ostream &f,size_t index)
 {
-  f.put(static_cast<int>(index>>24));
+  f.put(static_cast<int>((index>>24)&0xff));
   f.put(static_cast<int>((index>>16)&0xff));
   f.put(static_cast<int>((index>>8)&0xff));
   f.put(static_cast<int>(index&0xff));
@@ -240,26 +249,25 @@ static void writeInt(std::ostream &f,size_t index)
 
 static void writeString(std::ostream &f,const QCString &s)
 {
-  uint l = s.length();
-  for (uint i=0;i<l;i++) f.put(s[i]);
+  size_t l = s.length();
+  for (size_t i=0;i<l;i++) f.put(s[i]);
   f.put(0);
 }
 
 void SearchIndex::write(const QCString &fileName)
 {
-  size_t i;
   size_t size=4; // for the header
   size+=4*numIndexEntries; // for the index
   size_t wordsOffset = size;
   // first pass: compute the size of the wordlist
-  for (i=0;i<numIndexEntries;i++)
+  for (size_t i=0;i<numIndexEntries;i++)
   {
     const auto &wlist = m_index[i];
     if (!wlist.empty())
     {
       for (const auto &iw : wlist)
       {
-        int ws = iw.word().length()+1;
+        size_t ws = iw.word().length()+1;
         size+=ws+4; // word + url info list offset
       }
       size+=1; // zero list terminator
@@ -269,7 +277,7 @@ void SearchIndex::write(const QCString &fileName)
   // second pass: compute the offsets in the index
   size_t indexOffsets[numIndexEntries];
   size_t offset=wordsOffset;
-  for (i=0;i<numIndexEntries;i++)
+  for (size_t i=0;i<numIndexEntries;i++)
   {
     const auto &wlist = m_index[i];
     if (!wlist.empty())
@@ -296,7 +304,7 @@ void SearchIndex::write(const QCString &fileName)
   int count=0;
 
   // third pass: compute offset to stats info for each word
-  for (i=0;i<numIndexEntries;i++)
+  for (size_t i=0;i<numIndexEntries;i++)
   {
     const auto &wlist = m_index[i];
     if (!wlist.empty())
@@ -318,19 +326,19 @@ void SearchIndex::write(const QCString &fileName)
   }
 
   //printf("Total size %x bytes (word=%x stats=%x urls=%x)\n",size,wordsOffset,statsOffset,urlsOffset);
-  std::ofstream f(fileName.str(),std::ofstream::out | std::ofstream::binary);
+  std::ofstream f = Portable::openOutputStream(fileName);
   if (f.is_open())
   {
     // write header
     f.put('D'); f.put('O'); f.put('X'); f.put('S');
     // write index
-    for (i=0;i<numIndexEntries;i++)
+    for (size_t i=0;i<numIndexEntries;i++)
     {
       writeInt(f,indexOffsets[i]);
     }
     // write word lists
     count=0;
-    for (i=0;i<numIndexEntries;i++)
+    for (size_t i=0;i<numIndexEntries;i++)
     {
       const auto &wlist = m_index[i];
       if (!wlist.empty())
@@ -344,9 +352,9 @@ void SearchIndex::write(const QCString &fileName)
       }
     }
     // write extra padding bytes
-    for (i=0;i<padding;i++) f.put(0);
+    for (size_t i=0;i<padding;i++) f.put(0);
     // write word statistics
-    for (i=0;i<numIndexEntries;i++)
+    for (size_t i=0;i<numIndexEntries;i++)
     {
       const auto &wlist = m_index[i];
       if (!wlist.empty())
@@ -373,31 +381,6 @@ void SearchIndex::write(const QCString &fileName)
 
 }
 
-static std::mutex g_transferSearchIndexMutex;
-
-void SIDataCollection::transfer()
-{
-  if (Doxygen::searchIndex)
-  {
-    std::lock_guard<std::mutex> lock(g_transferSearchIndexMutex);
-    for (const auto &v : m_data)
-    {
-      if (std::holds_alternative<SIData_Word>(v))
-      {
-        const auto &d = std::get<SIData_Word>(v);
-        Doxygen::searchIndex->addWord(d.word,d.hiPrio);
-      }
-      else if (std::holds_alternative<SIData_CurrentDoc>(v))
-      {
-        const auto &d = std::get<SIData_CurrentDoc>(v);
-        Doxygen::searchIndex->setCurrentDoc(d.ctx,d.anchor,d.isSourceFile);
-      }
-    }
-  }
-  m_data.clear();
-}
-
-
 //---------------------------------------------------------------------------
 // the following part is for writing an external search index
 
@@ -455,6 +438,8 @@ static QCString definitionToName(const Definition *ctx)
         return "page";
       case Definition::TypeDir:
         return "dir";
+      case Definition::TypeModule:
+        return "module";
       default:
         break;
     }
@@ -464,9 +449,10 @@ static QCString definitionToName(const Definition *ctx)
 
 void SearchIndexExternal::setCurrentDoc(const Definition *ctx,const QCString &anchor,bool isSourceFile)
 {
+  std::lock_guard<std::mutex> lock(g_searchIndexMutex);
   QCString extId = stripPath(Config_getString(EXTERNAL_SEARCH_ID));
-  QCString baseName = isSourceFile ? (toFileDef(ctx))->getSourceFileBase() : ctx->getOutputFileBase();
-  QCString url = addHtmlExtensionIfMissing(baseName);
+  QCString url = isSourceFile ? (toFileDef(ctx))->getSourceFileBase() : ctx->getOutputFileBase();
+  addHtmlExtensionIfMissing(url);
   if (!anchor.isEmpty()) url+=QCString("#")+anchor;
   QCString key = extId+";"+url;
 
@@ -480,9 +466,25 @@ void SearchIndexExternal::setCurrentDoc(const Definition *ctx,const QCString &an
     {
       e.args = (toMemberDef(ctx))->argsString();
     }
+    else if (ctx->definitionType()==Definition::TypeGroup)
+    {
+      const GroupDef *gd = toGroupDef(ctx);
+      if (!gd->groupTitle().isEmpty())
+      {
+        e.name = filterTitle(gd->groupTitle());
+      }
+    }
+    else if (ctx->definitionType()==Definition::TypePage)
+    {
+      const PageDef *pd = toPageDef(ctx);
+      if (pd->hasTitle())
+      {
+        e.name = filterTitle(pd->title());
+      }
+    }
     e.extId = extId;
     e.url  = url;
-    it = m_docEntries.insert({key.str(),e}).first;
+    it = m_docEntries.emplace(key.str(),e).first;
     //printf("searchIndexExt %s : %s\n",qPrint(e->name),qPrint(e->url));
   }
   m_current = &it->second;
@@ -490,7 +492,8 @@ void SearchIndexExternal::setCurrentDoc(const Definition *ctx,const QCString &an
 
 void SearchIndexExternal::addWord(const QCString &word,bool hiPriority)
 {
-  if (word.isEmpty() || !isId(word[0]) || m_current==0) return;
+  std::lock_guard<std::mutex> lock(g_searchIndexMutex);
+  if (word.isEmpty() || !isId(word[0]) || m_current==nullptr) return;
   GrowBuf *pText = hiPriority ? &m_current->importantText : &m_current->normalText;
   if (pText->getPos()>0) pText->addChar(' ');
   pText->addStr(word);
@@ -499,14 +502,13 @@ void SearchIndexExternal::addWord(const QCString &word,bool hiPriority)
 
 void SearchIndexExternal::write(const QCString &fileName)
 {
-  std::ofstream t(fileName.str(),std::ofstream::out | std::ofstream::binary);
+  std::ofstream t = Portable::openOutputStream(fileName);
   if (t.is_open())
   {
     t << "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n";
     t << "<add>\n";
-    for (auto &kv : m_docEntries)
+    for (auto &[name,doc] : m_docEntries)
     {
-      SearchDocEntry &doc = kv.second;
       doc.normalText.addChar(0);    // make sure buffer ends with a 0 terminator
       doc.importantText.addChar(0); // make sure buffer ends with a 0 terminator
       t << "  <doc>\n";
@@ -542,15 +544,11 @@ void initSearchIndexer()
   bool externalSearch    = Config_getBool(EXTERNAL_SEARCH);
   if (searchEngine && serverBasedSearch)
   {
-    Doxygen::searchIndex = new SearchIndexIntf(externalSearch ? SearchIndexIntf::External : SearchIndexIntf::Internal);
-  }
-  else // no search engine or pure javascript based search function
-  {
-    Doxygen::searchIndex = 0;
+    Doxygen::searchIndex.setKind(externalSearch ? SearchIndexIntf::External : SearchIndexIntf::Internal);
   }
 }
 
 void finalizeSearchIndexer()
 {
-  delete Doxygen::searchIndex;
+  Doxygen::searchIndex.setKind(SearchIndexIntf::Disabled);
 }
