@@ -127,6 +127,8 @@ struct GetDefResult
   const FileDef      *fd=nullptr;
   const NamespaceDef *nd=nullptr;
   const GroupDef     *gd=nullptr;
+  const ConceptDef   *cnd=nullptr;
+  const ModuleDef    *modd=nullptr;
 };
 
 GetDefResult getDefs(const GetDefInput &input);
@@ -138,6 +140,7 @@ bool resolveRef(/* in */  const QCString &scName,
                 /* in */  bool inSeeBlock,
                 /* out */ const Definition **resContext,
                 /* out */ const MemberDef  **resMember,
+                /* in */  SrcLangExt lang,
                 /* in */  bool lookForSpecializations = TRUE,
                 /* in */  const FileDef *currentFile = nullptr,
                 /* in */  bool checkScope = FALSE
@@ -148,6 +151,7 @@ bool resolveLink(/* in */  const QCString &scName,
                  /* in */  bool inSeeBlock,
                  /* out */ const Definition **resContext,
                  /* out */ QCString &resAnchor,
+                 /* in */  SrcLangExt lang,
                  /* in */  const QCString &prefix=QCString()
                 );
 
@@ -253,6 +257,8 @@ QCString replaceAnonymousScopes(const QCString &s,const QCString &replacement=QC
 
 QCString convertNameToFile(const QCString &name,bool allowDots=FALSE,bool allowUnderscore=FALSE);
 
+QCString generateAnonymousAnchor(const QCString &fileName,int count);
+
 void extractNamespaceName(const QCString &scopeName,
                           QCString &className,QCString &namespaceName,
                           bool allowEmptyClass=FALSE);
@@ -340,7 +346,7 @@ QCString stripExtensionGeneral(const QCString &fName, const QCString &ext);
 
 QCString stripExtension(const QCString &fName);
 
-void replaceNamespaceAliases(QCString &scope,size_t i);
+//void replaceNamespaceAliases(QCString &scope,size_t i);
 
 int computeQualifiedIndex(const QCString &name);
 
@@ -427,7 +433,7 @@ QCString processMarkup(const QCString &s);
 bool protectionLevelVisible(Protection prot);
 
 QCString stripIndentation(const QCString &s);
-void stripIndentation(QCString &doc,const int indentationLevel);
+void stripIndentationVerbatim(QCString &doc,const int indentationLevel);
 
 QCString getDotImageExtension();
 
@@ -438,8 +444,8 @@ QCString extractDirection(QCString &docs);
 void convertProtectionLevel(
                    MemberListType inListType,
                    Protection inProt,
-                   int *outListType1,
-                   int *outListType2
+                   MemberListType *outListType1,
+                   MemberListType *outListType2
                   );
 
 bool mainPageHasTitle();
@@ -468,4 +474,9 @@ QCString projectLogoFile();
 
 void mergeMemberOverrideOptions(MemberDefMutable *md1,MemberDefMutable *md2);
 
+size_t updateColumnCount(const char *s,size_t col);
+
+QCString mangleCSharpGenericName(const QCString &name);
+QCString demangleCSharpGenericName(const QCString &name,const QCString &templArgs);
+
 #endif
