@@ -22,28 +22,14 @@
 
  @licend  The above is the entire license notice for the JavaScript code in this file
  */
-function SearchBox(name, resultsPath, inFrame, label)
-{
-  this.searchLabel = label;
-  this.DOMSearchField = function()
-  {  return document.getElementById("MSearchField");  }
-  this.DOMSearchBox = function()
-  {  return document.getElementById("MSearchBox");  }
-  this.OnSearchFieldFocus = function(isActive)
-  {
-    if (isActive)
-    {
+function SearchBox(name, resultsPath, extension) {
+  this.DOMSearchField = () => document.getElementById("MSearchField");
+  this.DOMSearchBox = () => document.getElementById("MSearchBox");
+  this.OnSearchFieldFocus = isActive => {
+    if (isActive) {
       this.DOMSearchBox().className = 'MSearchBoxActive';
-      var searchField = this.DOMSearchField();
-      if (searchField.value == this.searchLabel)
-      {
-        searchField.value = '';
-      }
-    }
-    else
-    {
+    } else {
       this.DOMSearchBox().className = 'MSearchBoxInactive';
-      this.DOMSearchField().value   = this.searchLabel;
     }
   }
 }
@@ -58,7 +44,7 @@ function getURLParameter(name) {
          ||[,""])[1].replace(/\+/g, '%20'))||null;
 }
 
-var entityMap = {
+const entityMap = {
   "&": "&amp;",
   "<": "&lt;",
   ">": "&gt;",
@@ -68,20 +54,19 @@ var entityMap = {
 };
 
 function escapeHtml(s) {
-  return String(s).replace(/[&<>"'\/]/g, function (s) {
+  return String(s).replace(/[&<>"'/]/g, function (s) {
     return entityMap[s];
   });
 }
 
 function searchFor(query,page,count) {
-  $.getJSON(serverUrl+"?cb=?",
-  {
+  $.getJSON(serverUrl+"?cb=?", {
     n:count,
     p:page,
     q:query
   },
   function(data) {
-    var results = $('#searchresults');
+    const results = $('#searchresults');
     $('#MSearchField').val(query);
     if (data.hits>0) {
       if (data.hits==1) {
@@ -89,9 +74,9 @@ function searchFor(query,page,count) {
       } else {
         results.html('<p>'+searchResultsText[2].replace(/\$num/,data.hits)+'</p>');
       }
-      var r='<table>';
+      let r='<table>';
       $.each(data.items, function(i,item){
-        var prefix = tagMap[item.tag];
+        let prefix = tagMap[item.tag];
         if (prefix) prefix+='/'; else prefix='';
         r+='<tr class="searchresult">'+
            '<td align="right">'+(data.first+i+1)+'.</td>'+
@@ -99,48 +84,38 @@ function searchFor(query,page,count) {
                 '<a href="'+escapeHtml(prefix+item.url)+
                 '">'+escapeHtml(item.name)+'</a>';
         if (item.type=="source") {
-          var l=item.url.match(/[1-9][0-9]*$/);
+          const l=item.url.match(/[1-9][0-9]*$/);
           if (l) r+=' at line '+parseInt(l[0]);
         }
         r+='</td>';
-        for (var i=0;i<item.fragments.length;i++)
-        {
+        for (let i=0;i<item.fragments.length;i++) {
           r+='<tr><td></td><td>'+item.fragments[i]+'</td></tr>';
         }
         r+='</tr>';
       });
       r+='</table>';
-      if (data.pages>1) // write multi page navigation bar
-      {
+      if (data.pages>1) { // write multi page navigation bar
         r+='<div class="searchpages">';
-        if (data.page>0)
-        {
-          r+='<span class="pages"><a href="javascript:searchFor(\''+escapeHtml(query)+'\','+(page-1).toString()+','+count.toString()+')">&laquo;</a></span>&nbsp;';
+        if (data.page>0) {
+          r+='<span class="pages"><a href="javascript:searchFor(\''+escapeHtml(query)+'\','+(page-1).toString()+','+count.toString()+')">&laquo;</a></span>&#160;';
         }
-        var firstPage = data.page-5;
-        var lastPage  = data.page+5;
-        if (firstPage<0)
-        {
+        let firstPage = data.page-5;
+        let lastPage  = data.page+5;
+        if (firstPage<0) {
           lastPage-=firstPage;
           firstPage=0;
         }
-        if (lastPage>data.pages)
-        {
+        if (lastPage>data.pages) {
           lastPage=data.pages;
         }
-        for(var i=firstPage;i<lastPage;i++)
-        {
-          if (i==data.page)
-          {
-            r+='<span class="pages"><b>'+(i+1).toString()+'</b></span>&nbsp;';
-          }
-          else
-          {
-            r+='<span class="pages"><a href="javascript:searchFor(\''+escapeHtml(query)+'\','+i.toString()+','+count.toString()+')">'+(i+1).toString()+'</a></span>&nbsp;';
+        for (let i=firstPage;i<lastPage;i++) {
+          if (i==data.page) {
+            r+='<span class="pages"><b>'+(i+1).toString()+'</b></span>&#160;';
+          } else {
+            r+='<span class="pages"><a href="javascript:searchFor(\''+escapeHtml(query)+'\','+i.toString()+','+count.toString()+')">'+(i+1).toString()+'</a></span>&#160;';
           }
         }
-        if (data.page+1<data.pages)
-        {
+        if (data.page+1<data.pages) {
           r+='<span class="pages"><a href="javascript:searchFor(\''+escapeHtml(query)+'\','+(page+1).toString()+','+count.toString()+')">&raquo;</a></span>';
         }
         r+='</div>';
