@@ -65,6 +65,9 @@ class Tester:
         self.test      = test
         self.update    = args.updateref
         self.config    = self.get_config()
+        if not 'objective' in self.config:
+            print("Test %s is missing the objective." % self.test)
+            sys.exit(1)
         self.test_name = '[%s]: %s' % (self.test,self.config['objective'][0])
         self.test_id   = self.test.split('_')[0]
         if self.update:
@@ -265,6 +268,7 @@ class Tester:
                     print(data,file=f)
         shutil.rmtree(self.test_out+'/out',ignore_errors=True)
         os.remove(self.test_out+'/Doxyfile')
+        os.remove(self.test_out+'/warnings.log')
         return True
 
     # check the relevant files of a doxygen run with the reference material
